# coding=utf-8
import time
from acfunsdk.source import routes, apis
from acfunsdk.exceptions import *


class AcComment:
    resource_type_map = {
        "1": 6,  # 番剧
        "2": 3,  # 视频稿件
        "3": 1,  # 文章稿件
        "10": 4,  # 动态
    }

    sourceId = None
    hot_comments = list()
    root_comments = list()
    sub_comments = dict()
    commentIds = list()
    commentsMap = dict()

    def __init__(self, acer, rtype: [str, int], rid: [str, int]):
        self.acer = acer
        self.sourceId = str(rid)
        self.sourceType = self.resource_type_map[str(rtype)]
        self.main = self.acer.acfun.resource(rtype, rid)

    def __repr__(self):
        return f"AcComment([ac{self.sourceId}] Σ{len(self.root_comments)})"

    @property
    def referer(self):
        return self.main.referer

    def _get_data(self, page: int = 1, api_name: str = 'comment'):
        assert api_name in ['comment', 'comment_floor']
        param = {
            "sourceId": self.sourceId,
            "sourceType": self.sourceType,
            "page": page,
            "pivotCommentId": 0,
            "newPivotCommentId": "",
            "t": str(time.time_ns())[:13],
            "supportZtEmot": True,
        }
        req = self.acer.client.get(apis[api_name], params=param)
        return req.json()

    def _get_sub(self, root_id, page: int = 1):
        param = {
            "sourceId": self.sourceId,
            "sourceType": self.sourceType,
            "rootCommentId": root_id,
            "page": page,
            "t": str(time.time_ns())[:13],
            "supportZtEmot": True
        }
        req = self.acer.client.get(apis['comment_subs'], params=param)
        return req.json()

    def get_all_comments(self):
        self.hot_comments = list()
        self.root_comments = list()
        self.sub_comments = dict()
        page = 1
        page_max = 10
        while page <= page_max:
            api_data = self._get_data(page)
            if api_data.get('result') != 0:
                print(api_data)
                break
            self.hot_comments.extend(api_data.get('hotComments', []))
            self.root_comments.extend(api_data.get('rootComments', []))
            self.sub_comments.update(api_data.get('subCommentsMap', {}))
            page_max = api_data.get('totalPage', page)
            page = api_data.get('curPage', 1)
            if page_max > 9:
                print(f"loading comments: {page}/{page_max}")
            page += 1

        for rid, sub_data in self.sub_comments.items():
            while sub_data['pcursor'] != "no_more":
                sub_page = self._get_sub(rid, sub_data['pcursor'])
                if 'subComments' not in sub_page:
                    sub_data['pcursor'] = "no_more"
                    break
                sub_data['subComments'].extend(sub_page['subComments'])
                if sub_page['curPage'] < sub_page['totalPage']:
                    sub_data['pcursor'] += 1
                else:
                    sub_data['pcursor'] = "no_more"
                time.sleep(0.1)

    def get_all_floors(self):
        first_page = self._get_data(1, 'comment_floor')
        self.commentIds = first_page['commentIds']
        self.commentsMap = first_page['commentsMap']
        page = first_page['curPage']
        page_max = first_page['totalPage']
        while page <= page_max:
            page += 1
            api_data = self._get_data(page, 'comment_floor')
            assert api_data['result'] == 0
            self.commentIds.extend(api_data['commentIds'])
            self.commentsMap.update(api_data['commentsMap'])
            time.sleep(0.5)

    @need_login
    def add(self, content: str, reply_id: [str, int, None] = None):
        form_data = {
            "sourceId": self.sourceId,
            "sourceType": self.sourceType,
            "content": content,
            "replyToCommentId": reply_id or "",
        }
        req = self.acer.client.post(apis['comment_add'], data=form_data,
                                    headers={'referer': self.referer})
        return req.json().get('result') == 0

    @need_login
    def delete(self, floor: int):
        this_comment = None
        for c in self.root_comments:
            if c.get('floor') == floor:
                this_comment = Comment(self.acer, c, self)
                break
        if this_comment is None:
            return False
        return this_comment.delete()

    def hot(self):
        datas = list()
        for c in self.hot_comments:
            datas.append(Comment(self.acer, c, self))
        return datas

    def list(self):
        datas = list()
        for c in self.root_comments:
            datas.append(Comment(self.acer, c, self))
        return datas

    def get(self, floor: int):
        for x in self.root_comments:
            if x['floor'] == floor:
                return Comment(self.acer, x, self)
        return None

    def find(self, cid: [int, str]):
        for x in self.root_comments:
            if x['commentId'] == int(cid):
                return Comment(self.acer, x, self)
        for x in self.sub_comments.values():
            for y in x.get('subComments', []):
                if y['commentId'] == int(cid):
                    return Comment(self.acer, y, self)
        return None


class Comment:
    resource_type = 6
    data = dict()
    ac_num = None

    def __init__(self, acer, data: dict, main_obj: AcComment):
        self.acer = acer
        self.data = data
        self.ac_num = self.data.get('sourceId')
        self.main_obj = main_obj

    def cdata(self, name, default=None):
        return self.data.get(name, default)

    def up(self):
        return self.acer.acfun.AcUp(self.cdata('userId'))

    def __repr__(self):
        content = self.data.get('content', '').replace('\r\n', '↲  ').replace('\n', '↲  ')
        return f"C([{self.cdata('commentId')}]#{self.cdata('floor')} {content} " \
               f"@{self.cdata('userName')}[{self.cdata('userId')}])"

    @property
    def referer(self):
        return f"{self.main_obj.referer}#ncid={self.data.get('commentId')}"

    @property
    def api_data(self):
        return {
            "sourceId": self.data.get('sourceId'),
            "sourceType": self.data.get('sourceType'),
            "commentId": self.data.get('commentId'),
        }

    @need_login
    def like(self):
        if self.cdata('isLiked') is False:
            req = self.acer.client.post(apis['comment_like'], data=self.api_data, headers={'referer': self.referer})
            return req.json().get('result') == 0
        return True

    @need_login
    def unlike(self):
        if self.cdata('isLiked') is True:
            req = self.acer.client.post(apis['comment_unlike'], data=self.api_data, headers={'referer': self.referer})
            return req.json().get('result') == 0
        return True

    @need_login
    def delete(self):
        req = self.acer.client.post(apis['comment_delete'], data=self.api_data, headers={'referer': self.referer})
        return req.json().get('result') == 0

    def report(self, crime: str, proof: str, description: str):
        return self.acer.acfun.AcReport.submit(
            self.referer, self.data.get('commentId'), self.resource_type,
            self.cdata('userId', "0"),
            crime, proof, description)



# coding=utf-8
import json
import time
from urllib import parse
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import routes, apis

__author__ = 'dolacmeo'


class AcSearch:
    reqID = 0
    result_raw = None
    result_obj = None
    hot_keywords = dict()
    sort_type = ['相关', '最多观看', '最多评论', '最多收藏', '最新发布']
    search_types = {
        'complex': 'complex-list',
        'video': 'video-list',
        'user': 'user-list',
        'article': 'article-list',
        'bgm': 'bangumi-list',
        'album': 'album-list',
    }

    def __init__(self, acer, keyword: [str, None] = None, s_type: [str, None] = None):
        self.acer = acer
        self._get_keywords()
        if keyword is None and 'searchKeywords' in self.hot_keywords:
            self.keyword = self.hot_keywords['searchKeywords'][0].get('keyword')
        else:
            self.keyword = keyword
        assert self.keyword
        if s_type in self.search_types:
            self.search_type = s_type
        else:
            self.search_type = 'complex'

    @property
    def referer(self):
        param = {
            "keyword": self.keyword,
            "type": self.search_type,
            "pCursor": 1,
        }
        return f"{routes['search']}?{parse.urlencode(param)}"

    def _get_data(self, page: int = 1, sortby: int = 1, channel_id: int = 0):
        if 1 > sortby > 5:
            sortby = 1
        param = {
            "keyword": self.keyword,
            "type": self.search_type,
            "quickViewId": self.search_types[self.search_type],
            "reqID": self.reqID,
            "ajaxpipe": 1,
            "pCursor": page,
            "sortType": sortby,
            "channelId": channel_id,
            "t": str(time.time_ns())[:13],
        }
        api_req = self.acer.client.get(apis['search'], params=param)
        assert api_req.text.endswith("/*<!-- fetch-stream -->*/")
        self.reqID += 1
        self.result_raw = json.loads(api_req.text[:-25])
        self.result_obj = Bs(self.result_raw.get('html', ''), 'lxml')

    def _get_keywords(self):
        api_req = self.acer.client.get(apis['search_keywords'])
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.hot_keywords = api_data

    def page(self, num=1, sortby: int = 1, channel_id: int = 0):
        self._get_data(num, sortby, channel_id)
        item_data = list()
        for item in self.result_obj.select('[class^=search-]'):
            if 'data-up-exposure-log' in item.attrs.keys():
                json_data = json.loads(item.attrs['data-up-exposure-log'])
                item_data.append(self.acer.acfun.AcUp(json_data['up_id']))
            else:
                json_data = json.loads(item.attrs['data-exposure-log'])
                if 'search-video' in item.attrs['class']:
                    item_data.append(self.acer.acfun.AcVideo(json_data['content_id'], {"title": json_data['title']}))
                elif 'search-article' in item.attrs['class']:
                    item_data.append(self.acer.acfun.AcArticle(json_data['content_id'], {"title": json_data['title']}))
                elif 'search-album' in item.attrs['class']:
                    item_data.append(self.acer.acfun.AcAlbum(json_data['content_id']))
                elif 'search-bangumi' in item.attrs['class']:
                    item_data.append(self.acer.acfun.AcBangumi(json_data['content_id']))
        return item_data







# coding=utf-8
import json
import httpx
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import routes, apis
from acfunsdk.page.utils import match1

__author__ = 'dolacmeo'


class AcDoodle:
    doodle_data = None
    doodle_title = None
    doodle_video = list()
    doodle_bg = list()
    doodle_image = list()
    doodle_text = list()
    doodle_button = list()
    doodle_vote = None
    page_obj = None

    def __init__(self, acer, doodle_id: str):
        self.acer = acer
        self.doodle_id = doodle_id
        if self.doodle_id.endswith(".html"):
            self.doodle_id = self.doodle_id[:-5]
        self.loading()

    def __repr__(self):
        return f"AcDoodle({self.doodle_title})"

    @property
    def referer(self):
        return f"{routes['doodle']}{self.doodle_id}.html"

    def loading(self):
        page_req = httpx.get(self.referer)
        self.page_obj = Bs(page_req.text, 'lxml')
        self.doodle_title = self.page_obj.select_one('title').text
        json_text = match1(page_req.text, r"(?s)__schema__\s*=\s*'(\{.*?\})';")
        self.doodle_data = json.loads(json_text.replace(r'\\"', r'\"'))
        root_id = self.doodle_data['app']['data']['rootContainer']
        main_block = None
        for item_id in self.doodle_data[root_id]['data']['children']:
            if self.doodle_data[item_id]['elementInfo']['label'] == 'Block':
                main_block = item_id
                block_bg = self.doodle_data[item_id]['values'].get("styles.background-image", {}).get("value")
                if block_bg:
                    self.doodle_bg.append(block_bg)
                break
        if main_block is None:
            return None
        for k in self.doodle_data[main_block]['data']['children']:
            ele = self.doodle_data[k]
            ele_name = ele['elementInfo']['label']
            ele_bg = ele['values'].get("styles.background-image", {}).get("value")
            if ele_bg:
                self.doodle_bg.append(ele_bg)
            if ele_name == 'PcAcfunVideo':
                self.doodle_video.append({
                    "raw": ele,
                    "name": ele_name,
                    "videoId": ele['values']['vid']['value'],
                    "resourceId": ele['values']['resourceId']['value'],
                })
            elif ele_name == 'PcCommonImage':
                self.doodle_image.append({
                    "raw": ele,
                    "name": ele['editorData']['name'],
                    "src": ele['values']['src']['value'],
                })
            elif ele_name == 'PcCommonText':
                self.doodle_text.append({
                    "raw": ele,
                    "name": ele['editorData']['name'],
                    "text": ele['values']['text']['value'],
                })
            elif ele_name == 'PcCommonButton':
                self.doodle_button.append({
                    "raw": ele,
                    "name": ele['editorData']['name'],
                    "url": ele['values']['url']['value'],
                    "action": ele['values']['action']['value'],
                })
            elif ele_name == 'PcAcfunVote':
                self.doodle_vote = {
                    "raw": ele,
                    "name": ele['editorData']['name'],
                    "voteId": ele['values']['voteId']['value'],
                    "voteToken": ele['values']['voteToken']['value'],
                }
        self.doodle_image = sorted(self.doodle_image, key=lambda x: x['raw']['values']['styles.offset-y']['value'])

    def vote_data(self):
        if self.doodle_vote is None:
            return None
        data = {
            "token": self.doodle_vote['voteToken'],
            "tags": "",
            "page": 1,
            "count": 10,
            "sort": 1,
            "title": ""
        }
        api_req = self.acer.client.post(apis['doodle_vote'], json=data)
        api_data = api_req.json()
        assert api_data.get("result") == 1
        return api_data

    def comment_feed(self, pcursor: [str, None] = None):
        form = {
            "objectId": f"{self.doodle_id}",
            "pageSize": "10",
            "subBiz": "jimu",
            "kpf": "PC_WEB",
            "kpn": "ACFUN_APP",
            "pcursor": pcursor or "",
        }
        api_req = self.acer.client.post(apis['doodle_comment'], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 1
        return api_data


if __name__ == '__main__':
    pass



# coding=utf-8
import math
from .utils import ms2time
from acfunsdk.source import routes, apis
from acfunsdk.exceptions import *


class AcDanmaku:
    ac_num = None
    vid = None
    danmaku_data = list()
    video_data = dict()

    def __init__(self, acer, video_data: dict):
        self.video_data = video_data
        self.ac_num = self.video_data.get('dougaId', self.video_data.get('bangumiId'))
        self.vid = self.video_data.get('currentVideoId', self.video_data.get('videoId'))
        self.acer = acer
        self._get_all_danmaku()

    def __repr__(self):
        return f"AcDanmaku([{self.vid}] Σ{len(self.danmaku_data)})"

    def _get_data(self, page=1, limit=200, sorttype=2, asc=True):
        param = {
            "resourceId": f"{self.vid}",
            "resourceType": "9",
            "enableAdvanced": True,
            "pcursor": f"{page}",
            "count": f"{limit}",
            "sortType": f"{sorttype}",
            "asc": asc,
        }
        req = self.acer.client.get(apis['danmaku'], params=param)
        return req.json()

    def _get_all_danmaku(self):
        self.danmaku_data = list()
        limit = 200
        page = 1
        more = 1
        while more > 0:
            api_data = self._get_data(page)
            total = api_data.get('totalCount', 1)
            self.danmaku_data.extend(api_data.get('danmakus', []))
            more = math.ceil(total / limit) - page
            page += 1

    def list(self):
        return [Danmaku(x, self.acer, self.video_data) for x in self.danmaku_data]

    @need_login
    def add(self, words, ms, color=16777215, mode=1, size=25):
        danmaku = {
            "size": size,  # 大* 中25 小16
            "mode": mode,
            "color": color,
            "position": ms,
            "body": words,
            "type": "bangumi" if "bangumiTitle" in self.video_data else "douga",
            "videoId": self.vid,
            "id": self.ac_num,
            "subChannelId": self.video_data.get('subChannelId'),
            "subChannelName": self.video_data.get('subChannelName'),
            "roleId": ""
        }
        req = self.acer.client.post(apis['danmaku_add'], data=danmaku, headers={"referer": f"{routes['index']}"})
        return req.json().get('result') == 0


class Danmaku:
    data = dict()
    video_data = dict()
    ac_num = None
    vid = None
    isLike = None

    def __init__(self, data: dict, acer=None, video_data: [dict, None] = None):
        self.data = data
        self.acer = acer
        if isinstance(video_data, dict):
            self.video_data = video_data
            self.vid = self.video_data.get('currentVideoId')
            self.ac_num = self.video_data.get('dougaId')

    def __repr__(self):
        return f"DM([{ms2time(self.position)}]#{self.danmakuId} {self.body} @{self.userId})"

    def __getattr__(self, item):
        if item in self.data.keys():
            return self.data.get(item)
        return super().__getattribute__(item)

    def up(self):
        return self.acer.acfun.AcUp(self.userId)

    @need_login
    def like(self):
        req = self.acer.client.post(apis['danmaku_like'], params={"danmakuId": self.danmakuId})
        self.isLike = req.json().get('result') == 0
        return self.isLike is True

    @need_login
    def like_cancel(self):
        req = self.acer.client.post(apis['danmaku_like_cancel'], params={"danmakuId": self.danmakuId})
        self.isLike = not (req.json().get('result') == 0)
        return self.isLike is False

    @need_login
    def block_words(self, word=None):
        req = self.acer.client.post(apis['danmaku_block_add'], params={
            "blockWordsType": 1,
            "blockWords": word or self.body
        })
        return req.json().get('result') == 0

    @need_login
    def block_acer(self):
        req = self.acer.client.post(apis['danmaku_block_add'], params={
            "blockWordsType": 2,
            "blockWords": self.userId
        })
        return req.json().get('result') == 0

    @need_login
    def block_words_delete(self):
        req = self.acer.client.post(apis['danmaku_block_delete'], params={
            "blockWordsType": 1,
            "blockWordsList": self.body
        })
        return req.json().get('result') == 0

    @need_login
    def block_acer_delete(self):
        req = self.acer.client.post(apis['danmaku_block_delete'], params={
            "blockWordsType": 2,
            "blockWordsList": self.userId
        })
        return req.json().get('result') == 0

    @need_login
    def report(self):
        req = self.acer.client.post(apis['danmaku_block_delete'], params={
            "reportedUserId": self.userId,
            "danmakuId": self.danmakuId,
            "body": self.body,
            "type": "douga",
            "id": self.ac_num,
            "videoId": self.vid,
            "subChannelId": self.video_data.get('channel', {}).get('id'),
            "subChannelName": self.video_data.get('channel', {}).get('name'),
        })
        return req.json().get('result') == 0



# coding=utf-8
import httpx
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import scheme, routes, apis
from .utils import url_complete

__author__ = 'dolacmeo'

AcFunWeixin_qr = "http://weixin.qq.com/r/6i7a3qnEcGEMKe29b3tu"
AcFunWeixin_qr_img = "https://cdnfile.aixifan.com/static/img/qrcode_e280b30.jpg"

AcFun_ico = "https://cdn.aixifan.com/ico/favicon.ico"
AcFun_logo = "https://ali-imgs.acfun.cn/kos/nlav10360/static/packageDownload/static/img/logo.e3f3277bac07179de334.png"
AcFun_login_bg = "https://static.yximgs.com/udata/pkg/acfun/loginbg.be7a2d2876ab48ed.png"
AcFun_danmaku_bg = [
    "https://ali-imgs.acfun.cn/kos/nlav10360/static/packageDownload/static/img/danmu-1.13b8f4cb7a790fdcfe19.png",
    "https://ali-imgs.acfun.cn/kos/nlav10360/static/packageDownload/static/img/danmu-2.7a0c666031b39fd1fa65.png",
]


class AcLink:

    def __init__(self, acer, url, title=None):
        self.acer = acer
        self.title = title
        self.url = url_complete(url)

    def __repr__(self):
        show_link = f" >> {self.url}" if self.url else ""
        return f"AcLink({self.title or ''}{show_link})"

    def container(self):
        return self.acer.get(self.url)


class AcImage:

    def __init__(self, acer, src, url=None, name=None):
        self.acer = acer
        self.src = url_complete(src)
        self.name = name
        self.url = url_complete(url)

    def __repr__(self):
        show_link = f" >> {self.url}" if self.url else ""
        return f"AcImg({self.name}[{self.src}]{show_link})"

    def container(self):
        return self.acer.get(self.url)


class AcHelp:
    banner = "https://ali-imgs.acfun.cn/kos/nlav10360/static/img/help-banner.740f1669.png"
    category = None
    hots = None

    def __init__(self):
        self.loading()

    @property
    def referer(self):
        return f"{routes['help']}"

    def loading(self):
        tab_req = httpx.post(apis['feedback_tab'], json={"appType": "acfun_m"})
        tab_data = tab_req.json()
        assert tab_data.get("result") == 1
        for x in tab_data.get("tabFaqs", []):
            if x['tab'] == 'all':
                self.category = {str(c['id']): c for c in x['faqs']}
            if x['tab'] == 'common':
                self.hots = x

    def tab(self, parent_id: str):
        if parent_id not in self.category.keys():
            return None
        children_req = httpx.post(apis['feedback_children'],
                                  json={"appType": "acfun_m", "parentId": parent_id})
        children_data = children_req.json()
        assert children_data.get("result") == 1
        return children_data.get("children")

    def get(self, question_id: str):
        question_req = httpx.post(apis['feedback_question'],
                                  json={"appType": "acfun_m", "questionId": question_id})
        question_data = question_req.json()
        assert question_data.get("result") == 1
        return question_data


class AcInfo:
    referer = f"{routes['info']}"

    about = {
        "url": "https://www.acfun.cn/info#page=about",
        "title": "关于AcFun弹幕视频网",
        "subtilte": "About Us",
        "content": [
            "AcFun弹幕视频网隶属于北京弹幕网络科技有限公司，是中国最具影响力的弹幕视频平台，"
            "也是全球最早上线的弹幕视频网站之一。“AcFun”原取意于“Anime Comic Fun”。"
            "自2007年6月6日成立以来，AcFun历经几年努力，从最初单一的视频站发展为现在的综合性弹幕视频网站，"
            "目前已是国内弹幕视频行业的领军品牌。",
            "AcFun从创立之初就坚持以“天下漫友是一家”为主旨，以“认真你就输了”为文化导向，倡导轻松欢快的亚文化。"
            "在几年中吸引了无数深爱宅文化的观众，也诞生了难以计数的知名原创视频作者。"
            "几年中由AcFun推广的无数优秀视频作品，深刻影响和改变了众多宅文化爱好者，"
            "它们中的大多数现已经成为网络经典。",
            "让更多人融入到弹幕视频的互动中去，让更多人理解宅文化的魅力所在，这就是几年来AcFun一直孜孜不倦追求的目标。"
            "随着时代变迁，AcFun也在不断进步和革新，以崭新的面貌，为所有热爱宅文化的观众们带来更完美的视听享受。",
        ]
    }
    contact = {
        "url": "https://www.acfun.cn/info#page=contact",
        "title": "联系我们",
        "subtilte": "Contact Us",
    }
    agreement = {
        "url": "https://www.acfun.cn/info#page=agreement",
        "title": "AcFun软件许可用户服务协议",
        "subtilte": "Agreement",
    }
    privacy = {
        "url": "https://www.acfun.cn/info#page=privacy",
        "title": "隐私权保护政策",
        "subtilte": "privacy",
    }
    children_privacy = {
        "url": "https://www.acfun.cn/info#page=children-privacy",
        "title": "儿童个人信息保护规则及监护人须知",
        "subtilte": "Children's Privacy",
    }
    children_guard = {
        "url": "https://www.acfun.cn/info#page=children-guard",
        "title": "儿童守护协议及监护人须知",
        "subtilte": "Children's Guard",
    }
    experience = {
        "url": "https://www.acfun.cn/info#page=experience",
        "title": "等级系统说明",
        "subtilte": "Experience System",
        "content": [
            "https://cdn.aixifan.com/dotnet/20130418/project/info/style/image/experience-1.jpg",
            "https://cdn.aixifan.com/dotnet/20130418/project/info/style/image/experience-2-2.jpg",
            "https://cdn.aixifan.com/dotnet/20130418/project/info/style/image/experience-3.png"
        ]
    }
    limit = {
        "url": "https://www.acfun.cn/info#page=limit",
        "title": "会员特权说明",
        "subtilte": "Experience System",
        "content": [
            "https://ali-imgs.acfun.cn/udata/pkg/acfun/member_level_desc.png"
        ]
    }


class AcReport:
    url = "https://www.acfun.cn/infringementreport"
    complaint_doc = "https://cdn.aixifan.com/downloads/AcFun%E4%B8%BE%E6%8A%A5%E7%94%B3%E8%AF%89%E8%A1%A8.doc"
    email = "ac-report@kuaishou.com"

    @staticmethod
    def submit(url: str, rid: str, rtype: str, uid: str, crime: str, proof: str, description: str):
        assert int(rtype) in [1, 2, 3, 4, 5, 6, 8, 10]
        crimes = ['色情', '血腥', '暴力', '猎奇', '政治', '辱骂', '广告', '挖坟', '剧透', '其他',
                  '话题不符', '少儿不宜', '未成年不良信息']
        assert int(crime) in range(1, len(crimes) + 1)
        assert uid.isdigit()
        assert len(url) >= 5
        assert len(proof) >= 5
        assert len(description) >= 5
        form = {
            "url": url,
            "resourceId": rid,
            "resourceType": rtype,
            "defendantUserId": uid,
            "crime": crime,
            "proof": proof,
            "description": description,
        }
        api_req = httpx.post(apis['report'], data=form)
        return api_req.json().get("result") == 0

    @staticmethod
    def feedback(title: str, content: str, tel: str):
        assert len(title) >= 5
        assert len(content) >= 5
        assert len(tel) == 11
        form = {"title": title, "content": content, "tel": tel}
        api_req = httpx.post(apis['feedback'], data=form)
        return api_req.json().get("result") == 0


class AcAcademy:
    url = "https://member.acfun.cn/academy"
    banner = "https://static.yximgs.com/udata/pkg/acfun-fe/up.cd94ec8275ced105.png"
    background = "https://tx2.a.kwimgs.com/udata/pkg/acfun/bg.e4bb289f.png"
    question_image = "https://tx2.a.kwimgs.com/udata/pkg/acfun/qadayi.b87cd018.png"
    answer_image = "https://tx2.a.kwimgs.com/udata/pkg/acfun/pc.5b40a899.png"

    acer_class = [
        {
            "resourceType": 2,
            "resourceId": 11772415,
            "title": "「萌新UP入门教程」第一期：迈出创作之路第一步",
            "coverUrl": "https://imgs.aixifan.com/voUp9bcRrP-yaURNn-FNZZRb-JvAJVj-NBZ3Un.jpg",
        },
        {
            "resourceType": 2,
            "resourceId": 11808491,
            "title": "「萌新UP入门教程」第二期：如何选择分区？",
            "coverUrl": "https://imgs.aixifan.com/S6aSTGHCo0-ny2mqe-aiuaM3-BZfMna-ArAVbi.jpg",
        },
        {
            "resourceType": 2,
            "resourceId": 11836309,
            "title": "「萌新UP入门教程」第三期：用户中心内有乾坤",
            "coverUrl": "https://imgs.aixifan.com/kicNLtKNf1-7B73Az-EbUjYj-2eYbYv-BBVnYb.jpg",
        },
        {
            "resourceType": 2,
            "resourceId": 11836337,
            "title": "「萌新UP入门教程」第四期：香蕉的秘密",
            "coverUrl": "https://imgs.aixifan.com/JxHjqEDGrs-BBRf2m-InEvqu-iANFBz-bqa6ba.jpg",
        },
        {
            "resourceType": 2,
            "resourceId": 11836377,
            "title": "「萌新UP入门教程」第五期：阿普学院是什么组织？",
            "coverUrl": "https://imgs.aixifan.com/YJaQxr4JHu-Q7jyE3-32y6bm-yyEbEb-E7viie.jpg",
        },
        {
            "resourceType": 2,
            "resourceId": 11836385,
            "title": "「萌新UP入门教程」第六期：审核二三事",
            "coverUrl": "https://imgs.aixifan.com/V5eSIu1M7m-aaEJnu-UBn67r-AvIryu-3URjqq.jpg",
        },
    ]

    @staticmethod
    def acer_teacher():
        form = {"courseType": "2", "pageSize": 40, "page": 1}
        api_req = httpx.post(apis['academy_tea'], data=form)
        api_data = api_req.json()
        return api_data.get("teacherList")


class AcDownload:
    _app_page_raw = None

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        return f"{routes['app']}"

    def _app_page_obj(self):
        if self._app_page_raw is None:
            page_req = self.acer.client.get(routes['app'])
            self._app_page_raw = Bs(page_req.text, 'lxml')
        return self._app_page_raw

    def emots(self):
        urls = list()
        emot_page = self.acer.client.get(routes['emot'])
        emot_obj = Bs(emot_page.text, 'lxml')
        for item in emot_obj.select('.emot-download'):
            title = item.select_one(".emot-name").text
            url = item.select_one("a.download-btn").attrs['href']
            details = item.select_one(".emot-detail-top")
            images = list()
            if details:
                for img in details.select('img'):
                    images.append(f"{scheme}:{img.attrs['src']}")
            urls.append({'title': title, 'url': f"{scheme}:{url}", 'details': images})
        return urls

    def Android_apk(self):
        api_req = self.acer.client.get(apis['app_download'])
        api_data = api_req.json()
        return api_data.get('url')

    def iOS_link(self):
        ios_link = self._app_page_obj().select_one(".app-info .download a.ios")
        return ios_link.attrs['href']

    def LiveMate_win(self):
        win_link = self._app_page_obj().select_one('.zbbl-info .download a.win')
        return win_link.attrs['href']

    def VirtualView_win(self):
        win_link = self._app_page_obj().select_one('.mbzs-info .download a.win')
        psd_link = self._app_page_obj().select_one('.mbzs-info .dis a.download-psd')
        return win_link.attrs['href'], psd_link.attrs['href']

    def FaceCatcher_win(self):
        api_res = self.acer.client.post(apis['face_catcher'], headers={
            "referer": "https://www.acfun.cn/face-catcher",
            "origin": "https://www.acfun.cn"
        })
        api_data = api_res.json()
        return api_data['downloadUrl'], api_data['psdUrl']

    def emot_zips(self):
        urls = list()
        api_res = self.acer.client.post(apis['emot'], headers={
            "referer": "https://www.acfun.cn/info/",
            "origin": "https://www.acfun.cn"
        })
        api_data = api_res.json()
        for em in api_data['emotionPackageList']:
            if em['downloadUrl']:
                urls.append({'url': em['downloadUrl'], 'filename': f"{em['name']}.zip"})
        return urls



# coding=utf-8
import json
import time
from bs4 import BeautifulSoup as Bs
from bs4.element import Tag
from acfunsdk.source import routes
from acfunsdk.page.utils import get_page_pagelets, match1, match_info, url_complete

__author__ = 'dolacmeo'

pagelets_name = {
    "pagelet_header": "顶栏",
    "pagelet_banner": "Banner",
    "pagelet_navigation": "导航栏",
    'pagelet_top_area': "置顶",
    'pagelet_monkey_recommend': "猴子推荐",
    'pagelet_live': "直播",
    'pagelet_spring_festival': "春季节日活动",
    'pagelet_list_banana': "香蕉榜",
    "pagelet_douga": "动画",
    "pagelet_game": "游戏",
    "pagelet_amusement": "娱乐",
    "pagelet_bangumi_list": "番剧",
    "pagelet_life": "生活",
    "pagelet_tech": "科技",
    "pagelet_dance": "舞蹈·偶像",
    "pagelet_music": "音乐",
    "pagelet_film": "影视",
    "pagelet_fishpond": "鱼塘",
    "pagelet_sport": "体育",
    "pagelet_footer": "页脚",
    "footer": "页脚",
}


class AcPagelet:
    pagelet_raw = None
    pagelet_id = None
    pagelet_obj = None
    pagelet_sp = (
        'pagelet_header',
        'pagelet_banner',
        'pagelet_navigation',
        'pagelet_top_area',
        'pagelet_monkey_recommend',
        'pagelet_live',
        'pagelet_spring_festival',
        'pagelet_list_banana',
        'pagelet_bangumi_list',
    )
    pagelets_area = (
        "pagelet_douga",  # 动画
        "pagelet_game",  # 游戏
        "pagelet_amusement",  # 娱乐
        "pagelet_life",  # 生活
        "pagelet_tech",  # 科技
        "pagelet_dance",  # 舞蹈·偶像
        "pagelet_music",  # 音乐
        "pagelet_film",  # 影视
        "pagelet_fishpond",  # 鱼塘
        "pagelet_sport",  # 体育
    )

    def __init__(self, acer, pagelet_data: [Tag, dict]):
        self.acer = acer
        self.pagelet_raw = pagelet_data
        if isinstance(pagelet_data, Tag):
            self.pagelet_id = pagelet_data.attrs['id']
            self.pagelet_obj = pagelet_data
            assert self.pagelet_id.startswith("pagelet_") or \
                   self.pagelet_id in ['footer']
        elif isinstance(pagelet_data, dict):
            self.pagelet_id = pagelet_data.get('id')
            self.pagelet_obj = Bs(pagelet_data.get('html', ""), "lxml")
        else:
            raise TypeError("pagelet_data allow bs4Tag or dict.")

    def __repr__(self):
        return f"AcIndex(#{self.pagelet_id} {pagelets_name[self.pagelet_id]})"

    def _index_header(self, obj=False):
        if self.pagelet_id != "pagelet_header":
            return None
        data = list()
        for link in self.pagelet_obj.select(".header-top-con > a,#header-guide a"):
            data.append({'url': url_complete(link.attrs['href']), 'text': link.text.strip()})
        return data

    def _index_banner(self, obj=False):
        if self.pagelet_id != "pagelet_banner":
            return None
        data = dict()
        banner_css_text = "\n".join(self.pagelet_raw.get('styles', [])).replace("\n", "")
        banner_css = match1(banner_css_text, r"\.page-top-banner \.banner-pic \{(?P<banner>[^}]*)}")
        assert banner_css is not None
        banner_image = match1(banner_css, r"background-image: url\('(?P<image>[^)]*)'\);")
        assert banner_image is not None
        data['image'] = banner_image
        data['url'] = self.pagelet_obj.select_one(".page-top-banner > a.banner-pic").attrs['href']
        data['title'] = self.pagelet_obj.select_one(".float-text").text
        if obj is True:
            return self.acer.acfun.AcImage(data['image'], data['url'], data['title'])
        return data

    def _index_navigation(self, obj=False):
        if self.pagelet_id != "pagelet_navigation":
            return None
        nav_links = list()
        for first in self.pagelet_obj.select(".first-item"):
            first_a = first.select_one("a.first-link")
            if obj is True:
                item = {'item': self.acer.get(url_complete(first_a.attrs['href'])), "children": []}
            else:
                item = {
                    "url": url_complete(first_a.attrs['href']),
                    "text": first_a.text.strip(),
                    "children": []
                }
            second_items = first.select(".second-item")
            for second in second_items:
                second_a = second.select_one("a.second-link")
                if obj is True:
                    child = {'item': self.acer.get(url_complete(second_a.attrs['href']))}
                else:
                    child = {
                        "url": url_complete(second_a.attrs['href']),
                        "text": second_a.text.strip()
                    }
                item['children'].append(child)
            nav_links.append(item)
        return nav_links

    def _index_top_area(self, obj=False):
        if self.pagelet_id != "pagelet_top_area":
            return None
        data = dict(slider=list(), items=list())
        for js_data in self.pagelet_raw.get('scripts', [])[0].split('\n'):
            if js_data.strip().startswith("window.sliderData = ["):
                data['slider'] = json.loads(js_data.strip()[20:-1])
        for video in self.pagelet_obj.select('a.recommend-video.log-item'):
            data['items'].append({
                'mediaid': video.attrs['data-mediaid'],
                'url': routes['video'] + video.attrs['data-mediaid'],
                'title': video.select_one('.video-title').text,
                'cover': video.select_one('img').attrs['src']
            })
        if obj is True:
            return dict(
                slider=[self.acer.acfun.AcImage(s['image'], s['link'], s['title']) for s in data['slider']],
                items=[self.acer.acfun.AcVideo(v['mediaid'], dict(title=v['title'])) for v in data['items']]
            )
        return data

    def _index_monkey_recommend(self, obj=False):
        if self.pagelet_id != "pagelet_monkey_recommend":
            return None
        videos = list()
        for video in self.pagelet_obj.select(".monkey-recommend-videos > .video-list > .monkey-video"):
            v_data = {
                'mediaid': video.attrs['data-mediaid'],
                'url': routes['video'] + video.attrs['data-mediaid'],
                'title': video.select_one('.monkey-video-title').text,
                'cover': video.select_one('.monkey-video-cover').img.attrs['src'],
                'up': video.select_one('.monkey-up-name').attrs['title'],
                'up_url': routes['index'] + video.select_one('.monkey-up-name').attrs['href'],
            }
            infos = match_info(video.select_one('.monkey-video-title').attrs['title'])
            if infos is not None:
                v_data.update(infos)
            videos.append(v_data)
        data = dict(items=videos)
        ad_img = self.pagelet_obj.select_one('.activity-box')
        data['ad'] = {
            'title': ad_img.attrs['data-title'],
            'url': ad_img.attrs['href'],
            'image': ad_img.img.attrs['src']
        }
        if obj is True:
            return dict(
                ad=self.acer.acfun.AcImage(data['ad']['image'], f"{data['ad']['url']}", data['ad']['title']),
                items=[
                    self.acer.acfun.AcVideo(v['mediaid'], dict(
                        title=v['title'], user=dict(id=v['up_url'][3:], name=v['up'])))
                    for v in videos
                ]
            )
        return data

    def _index_live(self, obj=False):
        if self.pagelet_id != "pagelet_live":
            return None
        videos = list()
        for video in self.pagelet_obj.select(".live-module-videos > .video-list > .live-video"):
            videos.append({
                'liveid': video.attrs['data-liveid'],
                'url': routes['live'] + video.attrs['data-liveid'],
                'title': video.select_one('.live-video-title').text,
                'cover': video.select_one('.live-video-cover').img.attrs['src'],
                'up': video.select_one('.live-video-up-name').attrs['title'],
                'up_avatar': video.select_one('.live-video-avatar').img.attrs['src'],
            })
        data = dict(items=videos)
        ad_img = self.pagelet_obj.select_one('.activity-box')
        data['ad'] = {
            'title': ad_img.attrs['data-title'],
            'url': ad_img.attrs['href'],
            'image': ad_img.img.attrs['src']
        }
        if obj is True:
            return dict(
                ad=self.acer.acfun.AcImage(data['ad']['image'], f"{data['ad']['url']}", data['ad']['title']),
                items=[self.acer.acfun.AcLiveUp(v['liveid']) for v in videos]
            )
        return data

    def _index_spring_festival(self, obj=False):
        if self.pagelet_id != "pagelet_spring_festival":
            return None
        return None

    def _index_list_banana(self, obj=False):
        if self.pagelet_id != "pagelet_list_banana":
            return None
        data = dict(d1=list(), d3=list(), d7=list(), article=dict())
        for rank in [('d1', '.day-list'), ('d3', '.three-day-list'), ('d7', '.week-list')]:
            for video in self.pagelet_obj.select(f".rank-left > {rank[1]} > .banana-video"):
                v_data = {
                    'mediaid': video.attrs['data-mediaid'],
                    'url': routes['video'] + video.attrs['data-mediaid'],
                    'title': video.select_one('.banana-video-title').text,
                    'cover': video.select_one('.banana-video-cover').img.attrs['src'],
                    'up': video.select_one('.banana-up-name').attrs['title'],
                    'up_url': routes['index'] + video.select_one('.banana-up-name').attrs['href'],
                    'banana_count': video.select_one('.banana-count').text,
                }
                infos = match_info(video.select_one('.banana-video-title').attrs['title'])
                if infos is not None:
                    v_data.update(infos)
                data[rank[0]].append(v_data)
        for article_tab in self.pagelet_obj.select('.rank-right .main-header-item'):
            tab_name = article_tab.select_one('.header-item-link').text
            tab_url = article_tab.select_one('.header-item-link').attrs['href']
            data['article'][tab_name] = dict(name=tab_name, url=tab_url, article=list())
            for index, article in enumerate(article_tab.select('.tab-main-content > li')):
                if index == 0:
                    this_article = {
                        'mediaid': article.attrs['data-mediaid'],
                        'url': routes['article'] + article.attrs['data-mediaid'],
                        'title': article.select_one('.main-content-block > a > p.block-title').text,
                        'headimg': article.select_one('img.block-img').attrs['src'],
                        'up': article.select_one('span.block-up').a.attrs['title'],
                        'up_url': routes['index'] + article.select_one('span.block-up').a.attrs['href'],
                        'commentCount': article.select_one('span.icon-comments').text
                    }
                    infos = match_info(article.select_one('.main-content-block > a > p.block-title').attrs['title'])
                else:
                    this_article = {
                        'mediaid': article.attrs['data-mediaid'],
                        'url': routes['article'] + article.attrs['data-mediaid'],
                        'title': article.a.text,
                    }
                    infos = match_info(article.a.attrs['title'])
                if infos is not None:
                    this_article.update(infos)
                data['article'][tab_name]['article'].append(this_article)
        if obj is True:
            obj_data = dict(d1=list(), d3=list(), d7=list(), article=dict())
            for rank in ['d1', 'd3', 'd7']:
                obj_data[rank] = [
                    self.acer.acfun.AcVideo(
                        v['mediaid'], dict(title=v['title'], user=dict(id=v['up_url'][3:], name=v['up'])))
                    for v in data[rank]
                ]
            for tab_name in data['article'].keys():
                obj_data['article'][tab_name] = [
                    self.acer.acfun.AcArticle(
                        a['mediaid'],
                        dict(title=a['title'], user=dict(id=a.get('up_url', '   ')[3:], name=a.get('up', ''))))
                    for a in data['article'][tab_name]['article']
                ]
            return obj_data
        return data

    def _index_bangumi_list(self, obj=False):
        if self.pagelet_id != "pagelet_bangumi_list":
            return None
        data = dict(schedule=list(), recommend=list(), anli=list())
        data['title'] = self.pagelet_obj.select_one('.area-header span.header-title').text
        data['icon'] = self.pagelet_obj.select_one('.area-header img.header-icon').attrs['src']
        data['url'] = routes['index'] + self.pagelet_obj.select_one('.header-right-more').attrs['href']
        if obj is True:
            data.update({
                'channel': self.acer.get(data['url']),
                'icon': self.acer.acfun.AcImage(data['icon'], data['url'], f"{data['title']}_icon")
            })
        for i, day in enumerate(self.pagelet_obj.select('.area-left .column-list .time-block')):
            day_list = list()
            for bangumi in day.select('.time-block .list-item'):
                if 'has-img' in bangumi.attrs['class']:
                    media_data = {
                        'mediaid': bangumi.attrs['data-mediaid'],
                        'albumid': bangumi.attrs['data-albumid'],
                        'url': routes['bangumi'] + bangumi.attrs['data-albumid'],
                        'cover': bangumi.a.img.attrs['src'],
                        'name': bangumi.select_one('a:nth-child(2) > b').text,
                        'recently': bangumi.p.text
                    }
                else:
                    media_data = {
                        'mediaid': bangumi.attrs['data-mediaid'],
                        'albumid': bangumi.attrs['data-albumid'],
                        'url': routes['bangumi'] + bangumi.attrs['data-albumid'],
                        'name': bangumi.a.b.text,
                        'recently': bangumi.a.p.text
                    }
                if obj is True:
                    day_list.append(self.acer.acfun.AcBangumi(media_data['mediaid']))
                else:
                    day_list.append(media_data)
            data['schedule'].append(day_list)
        for goood in self.pagelet_obj.select('.area-left .block-list .block-list-item'):
            media_data = {
                'mediaid': goood.attrs['data-mediaid'],
                'albumid': goood.attrs['data-albumid'],
                'url': routes['bangumi'] + goood.attrs['data-albumid'],
                'cover': goood.select_one('.block-img img').attrs['src'],
                'name': goood.select_one('.block-list-title > b > a').text,
                'follow': goood.select_one('.block-list-title > p > i.fr').text
            }
            if obj is True:
                data['recommend'].append(self.acer.acfun.AcBangumi(media_data['mediaid']))
            else:
                data['recommend'].append(media_data)
        for block in self.pagelet_obj.select('.area-right .season-rec'):
            media_data = {
                'mediaid': block.attrs['data-mediaid'],
                'albumid': block.attrs['data-albumid'],
                'url': routes['bangumi'] + block.attrs['data-albumid'],
                'cover': block.img.attrs['src']
            }
            if obj is True:
                data['anli'].append(self.acer.acfun.AcBangumi(media_data['mediaid']))
            else:
                data['anli'].append(media_data)
        return data

    def _index_pagelet_left_info(self, obj=False):
        data = dict(title="", icon="", links=list(), url=None)
        data['title'] = self.pagelet_obj.select_one('.module-left-header span.header-title').text
        data['icon'] = self.pagelet_obj.select_one('.module-left-header img.header-icon').attrs['src']
        for link in self.pagelet_obj.select('.link-container a'):
            href = ("" if link.attrs['href'].startswith('http') else routes['index']) + link.attrs['href']
            data['links'].append({'url': href, 'title': link.text})
        data['url'] = routes['index'] + self.pagelet_obj.select_one('.header-right-more').attrs['href']
        if obj is True:
            return {
                'channel': self.acer.get(data['url']),
                'links': [self.acer.get(x['url'], x['title']) for x in data['links']],
                'icon': self.acer.acfun.AcImage(data['icon'], data['url'], f"{data['title']}_icon")
            }
        return data

    def _index_pagelet_right_rank(self, obj=False):
        data = dict(rank=dict(d1=list(), d3=list(), d7=list()))
        for index, rank in enumerate(self.pagelet_obj.select('.list-content-videos')):
            for rank_item in rank.select('.log-item'):
                rank_type = f'd{"137"[index]}'
                if 'video-item-big' in rank_item['class']:
                    rank_data = {
                        'mediaid': rank_item.attrs['data-mediaid'],
                        'url': routes['video'] + rank_item.attrs['data-mediaid'],
                        'title': rank_item.select_one('.video-title').text,
                        'cover': rank_item.select_one('.block-left > a > img').attrs['src'],
                        'up': rank_item.select_one('.video-up').attrs['title'],
                        'up_url': routes['index'] + rank_item.select_one('.video-up').attrs['href'],
                        'viewCount': rank_item.select_one('.video-info > .icon-view-player').text,
                        'commentCount': rank_item.select_one('.video-info > .icon-comments').text
                    }
                    infos = match_info(rank_item.select_one('.video-title').attrs['title'])
                else:
                    rank_data = {
                        'mediaid': rank_item.attrs['data-mediaid'],
                        'url': routes['video'] + rank_item.attrs['data-mediaid'],
                        'title': rank_item.a.text,
                    }
                    infos = match_info(rank_item.a.attrs['title'])
                if infos is not None:
                    rank_data.update(infos)
                data['rank'][rank_type].append(rank_data)
        data['rank']['url'] = routes['index'] + self.pagelet_obj.select_one('.ranked-list > .more').attrs['href']
        if obj is True:
            obj_data = dict(d1=list(), d3=list(), d7=list())
            for k in obj_data.keys():
                obj_data[k] = [
                    self.acer.acfun.AcVideo(v['mediaid'], dict(title=v['title']))
                    for v in data['rank'][k]
                ]
            return dict(rank=obj_data)
        return data

    def _index_pagelet(self, obj=False):
        data = dict(items=list())
        for video in self.pagelet_obj.select('div[class^="video-list-"] > div'):
            if 'big-image' in video['class']:
                v_data = {
                    'mediaid': video.a.attrs['href'][5:],
                    'url': routes['index'] + video.a.attrs['href'],
                    'title': video.select_one('.title').text,
                    'cover': video.select_one('.cover > img').attrs['src'],
                    'duration': video.select_one('.video-time').text,
                }
                infos = match_info(video.select_one('.title').attrs['title'])
            else:
                this_video = video.select_one('.normal-video')
                v_data = {
                    'mediaid': this_video.attrs['data-mediaid'],
                    'url': routes['video'] + this_video.attrs['data-mediaid'],
                    'title': this_video.select_one('.normal-video-title').text,
                    'cover': this_video.select_one('.normal-video-cover').img.attrs['src'],
                    'duration': this_video.select_one('.video-time').text,
                    'viewCount': this_video.select_one('.normal-video-info > .icon-view-player').text,
                    'danmakuCount': this_video.select_one('.normal-video-info > .icon-danmu').text
                }
                infos = match_info(this_video.select_one('.normal-video-title').attrs['title'])
            if infos is not None:
                v_data.update(infos)
            if obj is True:
                data['items'].append(self.acer.acfun.AcVideo(v_data['mediaid'], dict(title=v_data['title'])))
            else:
                data['items'].append(v_data)
        data.update(self._index_pagelet_left_info(obj))
        data.update(self._index_pagelet_right_rank(obj))
        return data

    def _footer(self, obj=False):
        if self.pagelet_id != "footer":
            return None
        data = dict(links=dict(), infos=list(), copyright=dict())
        for link_group in self.pagelet_obj.select('.footer-nav > div'):
            for i, tag in enumerate(link_group.select('h5')):
                those_links = list()
                for link in link_group.select(f'p:nth-of-type({i + 1}) a'):
                    if 'href' in link.attrs:
                        this_link = {
                            'title': link.text,
                            'url': link.attrs['href']
                        }
                        if obj is True:
                            those_links.append(self.acer.acfun.AcLink(**this_link))
                        else:
                            those_links.append(this_link)
                    else:
                        this_link = {
                            'name': link.get_text(),
                            'src': link.select_one('img').attrs['src']
                        }
                        if obj is True:
                            those_links.append(self.acer.acfun.AcImage(**this_link))
                        else:
                            those_links.append(this_link)
                data['links'][tag.text] = those_links
        for info in self.pagelet_obj.select('.footer-link a'):
            this_link = {
                'title': info.text.strip(),
                'url': info.attrs['href']
            }
            if obj is True:
                data['infos'].append(self.acer.acfun.AcLink(**this_link))
            else:
                data['infos'].append(this_link)
        for ac in self.pagelet_obj.select_one('.footer-link > div:last-child').select('span'):
            if ": " in ac.text:
                k, v = ac.text.split(": ")
                data['copyright'][k] = v
        return data

    def to_dict(self, obj=False):
        if self.pagelet_id == 'footer':
            return self._footer(obj)
        elif self.pagelet_id in self.pagelet_sp:
            func_name = self.pagelet_id.replace("pagelet_", "_index_")
            return getattr(self, func_name)(obj)
        elif self.pagelet_id in self.pagelets_area:
            return self._index_pagelet(obj)
        return None


class AcIndex:
    pagelets_from_page = [
        "pagelet_banner",  # banner
        "pagelet_navigation",  # 导航栏
        'pagelet_top_area',  # 置顶
        'pagelet_monkey_recommend',  # 猴子推荐
        'pagelet_live',  # 直播
        'pagelet_spring_festival',  # 春季节日活动
        'pagelet_list_banana',  # 香蕉榜
        'footer',  # 页脚
    ]
    pagelets_from_api = [
        "pagelet_header",  # 顶栏
        "pagelet_douga",  # 动画
        "pagelet_game",  # 游戏
        "pagelet_amusement",  # 娱乐
        "pagelet_bangumi_list",  # 番剧
        "pagelet_life",  # 生活
        "pagelet_tech",  # 科技
        "pagelet_dance",  # 舞蹈·偶像
        "pagelet_music",  # 音乐
        "pagelet_film",  # 影视
        "pagelet_fishpond",  # 鱼塘
        "pagelet_sport",  # 体育
    ]
    index_obj = None
    index_pagelets = []
    nav_data = dict()

    def __init__(self, acer=None):
        self.acer = acer
        self._get_index()

    def __repr__(self):
        return "AcIndex(AcFun弹幕视频网)"

    def _get_index(self):
        req = self.acer.client.get(routes['index'])
        self.index_obj = Bs(req.text, 'lxml')
        self.index_pagelets = get_page_pagelets(self.index_obj)

    def _get_pagelet_inner(self, area: [str, None] = None):
        datas = dict()
        for js in self.index_obj.select("script"):
            if js.text.startswith("bigPipe.onPageletArrive"):
                data = json.loads(js.text[24:-2])
                datas[data['id']] = data
        datas['footer'] = self.index_obj.select_one('#footer')
        if isinstance(area, str):
            return datas.get(area)
        return datas

    def _get_pagelet_api(self, area):
        assert area in self.pagelets_from_api
        param = {
            "pagelets": area, "reqID": 0, "ajaxpipe": 1,
            "t": str(time.time_ns())[:13]
        }
        req = self.acer.client.get(routes['index'] + '/', params=param)
        if req.text.endswith("/*<!-- fetch-stream -->*/"):
            return json.loads(req.text[:-25])
        return req.json()

    def nav_list(self):
        navs = list()
        for cid in self.acer.nav_data.keys():
            navs.append(self.acer.acfun.AcChannel(cid))
        return navs

    def get(self, area: str, obj=False):
        if area != 'footer' and not area.startswith("pagelet_"):
            area = "pagelet_" + area
        if area in self.pagelets_from_page:
            raw_data = self._get_pagelet_inner(area)
        elif area in self.pagelets_from_api:
            raw_data = self._get_pagelet_api(area)
        else:
            raise ValueError('area not support')
        acp = AcPagelet(self.acer, raw_data)
        return acp.to_dict(obj)



# coding=utf-8
import json
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import routes, apis
from acfunsdk.page.utils import ms2time, get_channel_info, get_page_pagelets, match1

__author__ = 'dolacmeo'


class AcVideo:
    resource_type = 2
    ac_num = None
    page_obj = None
    page_pagelets = []
    video_data = dict()
    vid = None
    is_404 = False

    def __init__(self, acer, ac_num: [str, int], video_data: [dict, None] = None):
        if isinstance(ac_num, str) and ac_num.startswith('ac'):
            ac_num = ac_num[2:]
        self.ac_num = str(ac_num)
        self.part_num = 1
        self.acer = acer
        if isinstance(video_data, dict):
            self.video_data = video_data
        if "_" in self.ac_num:
            self.ac_num, self.part_num = self.ac_num.split('_')
        self.loading()

    @property
    def referer(self):
        if int(self.part_num) == 1:
            return f"{routes['video']}{self.ac_num}"
        return f"{routes['video']}{self.ac_num}_{self.part_num}"

    @property
    def share_url(self):
        return self.referer

    @property
    def mobile_url(self):
        return f"https://scan.acfun.cn/vd/{self.ac_num}"

    def __repr__(self):
        if self.is_404:
            return f"AcVideo([ac{self.ac_num}]咦？世界线变动了。看看其他内容吧~)"
        title = self.video_data.get('title', "")
        title = title if len(title) < 28 else title[:27] + ".."
        user_name = self.video_data.get('user', {}).get('name', "") or self.video_data.get('user', {}).get('id', "")
        user_txt = "" if len(user_name) == 0 else f" @{user_name}"
        duration = self.video_data.get('durationMillis', 0)
        duration_txt = "" if duration == 0 else f"[{ms2time(duration)}]"
        return f"AcVideo([ac{self.ac_num}]{duration_txt}{title}{user_txt})".encode(errors='replace').decode()

    def loading(self):
        req = self.acer.client.get(self.referer)
        self.is_404 = req.status_code // 100 != 2
        if self.is_404:
            return False
        self.page_obj = Bs(req.text, 'lxml')
        json_text = match1(req.text, r"(?s)videoInfo\s*=\s*(\{.*?\});")
        self.video_data = json.loads(json_text)
        self.video_data.update(get_channel_info(req.text))
        self.vid = self.video_data.get("currentVideoId")
        self.page_pagelets = get_page_pagelets(self.page_obj)
        staff_data = self.staff()
        if staff_data is not None:
            self.video_data['staffInfos'] = staff_data.get('staffInfos')
            self.video_data['upInfo'] = staff_data.get('upInfo')

    @property
    def video_list(self):
        return self.video_data.get('videoList', [])

    def video_scenes(self):
        form_data = {"resourceId": self.ac_num, "resourceType": self.resource_type,
                     "videoId": self.video_data.get('currentVideoId')}
        api_req = self.acer.client.post(apis['video_scenes'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        if api_data.get("spriteVtt") is None:
            return None
        pos_data = list()
        sprite_data = api_data.get("spriteVtt", "").split("\n\n")[1:]
        sprite_img = sprite_data[0].split("\n")[1].split("#")[0]
        for line in sprite_data:
            pos, img_url = line.split("\n")
            pos_s, pos_e = pos.split(" --> ")
            _, xywh = img_url.split("#xywh=")
            pos_data.append([pos_s, pos_e, xywh])
        return {"sprite_image": sprite_img, "pos": pos_data}

    def video_hotspot(self):
        form_data = {"resourceId": self.ac_num, "resourceType": self.resource_type}
        api_req = self.acer.client.post(apis['video_hotspot'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        return api_data.get("hotSpotDistribution")

    def set_video(self, num=1):
        assert num <= len(self.video_list)
        self.part_num = num
        self.loading()
        return True

    def get_ksPlayJson(self, video_id: [str, int, None] = None):
        param = {"resourceId": self.ac_num, "resourceType": 2}
        if video_id is not None:
            param['videoId'] = video_id
        api_req = self.acer.client.get(apis['video_ksplay'], params=param)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        return api_data.get("playInfo")

    def up(self):
        user = self.video_data.get('user', {})
        return self.acer.acfun.AcUp(user.get("id"))

    def staff(self):
        if self.video_data.get('staffContribute') is not True:
            return None
        form_data = {"resourceId": self.ac_num, "resourceType": self.resource_type}
        api_req = self.acer.client.post(apis['getStaff'], data=form_data)
        api_data = api_req.json()
        return api_data

    def danmaku(self):
        return self.acer.acfun.AcDanmaku(self.video_data)

    def comment(self):
        return self.acer.acfun.AcComment(self.ac_num, 3)

    def like(self):
        return self.acer.like_add(self.ac_num, 2)

    def like_cancel(self):
        return self.acer.like_delete(self.ac_num, 2)

    def favorite_add(self, folder_id: [str, None] = None):
        return self.acer.favourite.add(self.ac_num, 9, folder_id)

    def favorite_cancel(self, folder_id: [str, None] = None):
        return self.acer.favourite.cancel(self.ac_num, 9, folder_id)

    def banana(self, count: int):
        return self.acer.throw_banana(self.ac_num, self.resource_type, count)

    # 一键奥里给！
    def aoligei(self, danmu: bool = False, comment: bool = False):
        """ 赞 藏 蕉 弹 评 """
        """ 👍 🔖 🍌 🌠 💬 """
        print(self.like())  # 👍 点赞
        print(self.favorite_add())  # 🔖 收藏
        print(self.banana(5))  # 🍌 投蕉
        if danmu is True:  # 🌠 发弹幕
            self.danmaku().add("棒棒棒~加油哦~", 0)
        if comment is True:  # 💬 留言
            self.comment().add('<p><font color="#ff0000">棒棒棒~加油哦~</font></p>'
                               '<p><font color="#c4bd97">from  acfunSDK</font></p>')
        print(f" 赞 藏 蕉 弹 评 \n 👍 🔖 🍌 🌠 💬 \n 分享：{self.referer}?shareUid={self.acer.uid}")
        return True

    def report(self, crime: str, proof: str, description: str):
        return self.acer.acfun.AcReport.submit(
            self.referer, self.ac_num, self.resource_type,
            self.video_data.get("user", {}).get("id", "0"),
            crime, proof, description)



# coding=utf-8
import json
from acfunsdk.source import routes, apis
from acfunsdk.page.utils import thin_string, limit_string, match1

__author__ = 'dolacmeo'


class AcMoment:
    resource_type = 10
    raw_data = dict()
    like_count = 0
    banana_count = 0
    comment_count = 0
    linked = None

    def __init__(self, acer, am_num: [int, str]):
        self.acer = acer
        self.am_num = int(am_num)
        self.loading()
        if self.rtype in range(1, 5):
            self.linked = self.acer.acfun.resource(self.rtype, self.rid)
        elif self.rtype == 10:
            rsource = self.raw_data.get('repostSource')
            if rsource is not None:
                self.linked = AcMoment(self.acer, self.raw_data.get('repostSource'))

    def loading(self):
        page_req = self.acer.client.get(f"{routes['moment']}{self.am_num}")
        json_text = match1(page_req.text, r"(?s)__INITIAL_STATE__\s*=\s*(\{.*?\});")
        self.raw_data = json.loads(json_text).get('moment', {}).get('moment')

    @property
    def tag_rtype(self):
        return self.raw_data.get('tagResourceType')

    @property
    def rtype(self):
        return self.raw_data.get('resourceType')

    @property
    def rid(self):
        return self.raw_data.get('resourceId')

    @property
    def mtype(self):
        return self.raw_data.get('momentType')

    @property
    def text_without_ubb(self):
        return self.raw_data.get("replaceUbbText")

    def __repr__(self):
        if self.tag_rtype is None and self.rtype == 10:
            return f"AcMoment({limit_string(self.text_without_ubb, 7)} {self.linked})"
        if self.tag_rtype in [1, 2, 10]:
            name = self.raw_data.get('user', {}).get('userName', '')
            return f"AcMoment(@{name} | {str(self.linked)})".encode(errors='replace').decode()
        elif self.tag_rtype == 3:
            name = self.raw_data.get('user', {}).get('userName', '')
            text = self.raw_data.get('moment', {}).get('replaceUbbText', '')
            # text = re.sub('\[emot=acfun,\d+/]', '', thin_string(text))
            text = thin_string(text)
            img_count = self.raw_data.get('discoveryResourceFeedShowImageCount', 0)
            img_count = f"【图x{img_count}】" if img_count > 0 else ""
            link = "" if self.linked is None else f" | {str(self.linked)}"
            return f"AcMoment(@{name}: {img_count}{text}{link})".encode(errors='replace').decode()
        return f"AcMoment()"

    @property
    def referer(self):
        return f"{routes['moment']}{self.am_num}"

    def up(self):
        user = self.raw_data.get('user')
        return self.acer.acfun.AcUp(user.get("id"))

    def comment(self):
        return self.acer.acfun.AcComment(self.resource_type, self.rid)

    def banana(self, count: int):
        return self.acer.throw_banana(self.rid, self.resource_type, count)

    def report(self, crime: str, proof: str, description: str):
        return self.acer.acfun.AcReport.submit(
            self.referer, self.am_num, self.resource_type,
            self.raw_data.get('user').get('id', "0"),
            crime, proof, description)


class MyMoment:
    cursor = "0"
    limit = 10
    moment_data = list()
    resourceTypes = 0
    rts = {  # 0综合 2视频 3文章 10动态
        'all': 0,
        'video': 2,
        'article': 3,
        'moment': 10
    }

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        rtypes = {
            "0": "?tab=all",
            "2": "?tab=video",
            "3": "?tab=article",
        }
        query = rtypes.get(str(self.resourceTypes), "")
        return f"{routes['feeds']}{query}"

    def set_tab(self, tab: str = 'all'):
        new = self.rts.get(tab, 0)
        if new != self.resourceTypes:
            self.resourceTypes = new
            self.cursor = "0"
            self.moment_data = list()
            return True
        return False

    def feed(self, limit: int = 10, refresh: bool = False):
        if refresh is True:
            self.cursor = "0"
            self.moment_data = list()
        param = {
            "useWebp": False,
            "pcursor": self.cursor,
            "count": limit or self.limit,
            "resourceTypes": self.resourceTypes,
        }
        api_req = self.acer.client.get(apis['follow_feed'], params=param)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.cursor = api_data.get('pcursor', self.cursor)
            self.moment_data.extend(api_data.get('feedList', []))
        return [AcMoment(self.acer, x) for x in api_data.get('feedList', [])]



# coding=utf-8
import re
import time
import json
from bs4 import BeautifulSoup as Bs
from .utils import thin_string
from acfunsdk.source import scheme, routes, apis


class Message:
    intro = None
    create_at = None
    whom = None
    raw_data = None
    raw_link = None

    def __init__(self, acer, **kwargs):
        self.acer = acer
        self.raw_data = kwargs
        self.raw_link = kwargs.get('content_url')
        self.intro = kwargs.get('intro')
        self.create_at = kwargs.get('create_at')
        if 'uid' in kwargs:
            self.whom = {'userId': kwargs.get('uid'), 'name': kwargs.get('username')}

    def __repr__(self):
        return f"AcMsg({self.intro})"

    def user(self):
        if self.whom is None:
            return None
        return self.acer.acfun.AcUp(self.whom)


class ReplyMsg(Message):

    def __init__(self, acer, **kwargs):
        self.content_title = kwargs.get('content_title', '')
        self.replied = kwargs.get('replied', '')
        self.content = kwargs.get('content', '')
        self.ncid = kwargs.get('ncid', '')
        self.username = kwargs.get('username', '')
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcReply({self.content_title}—— {self.content} @{self.username})"

    def content(self):
        return self.acer.get(self.raw_data.get('content_url'))

    def replay(self):
        comments = self.content().comment()
        return comments.find(self.raw_data.get('ncid'))


class LikeMsg(ReplyMsg):

    def __init__(self, acer, **kwargs):
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcLike({self.replied} | @{self.username})"


class AtMsg(ReplyMsg):

    def __init__(self, acer, **kwargs):
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcMsg({self.raw_link}#ncid={self.ncid} | @{self.username})"


class GiftMsg(Message):
    banana = 0

    def __init__(self, acer, **kwargs):
        self.content_title = kwargs.get('content_title', '')
        self.username = kwargs.get('username', '')
        self.banana = kwargs.get('banana', 0)
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcGift({self.content_title} | 🍌x{self.banana} @{self.username})"


class NoticeMsg(Message):

    def __init__(self, acer, **kwargs):
        self.content_title = kwargs.get('content_title', '')
        super().__init__(acer, **kwargs)

    def __repr__(self):
        return f"AcNotice({self.content_title} >>> {self.raw_link})"


class SystemMsg(Message):
    classify = None

    def __init__(self, acer, **kwargs):
        super().__init__(acer, **kwargs)

    def __repr__(self):
        if 'up' in self.raw_data and '关注了你' in self.raw_data.get('intro', ''):
            link = self.raw_data.get('up', [])
            return f"AcFans(+1 | @{link[0]})"
        elif 'video' in self.raw_data and '已通过审核' in self.raw_data.get('intro', ''):
            link = self.raw_data.get('video', [])
            return f"AcPass({link[0]} 已通过审核)"
        elif 'article' in self.raw_data and '已通过审核' in self.raw_data.get('intro', ''):
            link = self.raw_data.get('article', [])
            return f"AcPass({link[0]} 已通过审核)"
        elif '有人收藏了你的' in self.raw_data.get('intro', ''):
            if 'video' in self.raw_data:
                link = self.raw_data.get('video', [])
            elif 'article' in self.raw_data:
                link = self.raw_data.get('article', [])
            else:
                return f"AcMsg({self.intro})"
            return f"AcStar(+1 | {link[0]})"
        return f"AcMsg({self.intro})"


class MyMessage:
    req_count = 0
    means = {
        'new_comment': '评论',
        'at_notify': '@我',
        'new_comment_like': '点赞',
        'new_gift': '礼物',
        'new_content_notify': '站内公告',
        'new_system_notify': '系统通知',
    }

    def __init__(self, acer):
        self.acer = acer

    def __repr__(self):
        msg = list()
        for k, v in self.unread.items():
            if v > 0:
                msg.append(f"{self.means[k]}[{v}]")
        return f"MyMessage({''.join(msg)})"

    @property
    def unread(self):
        if self.acer.is_logined is False:
            return None
        api_req = self.acer.client.get(apis['unread'])
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        return api_data.get("unReadCount")

    def _get_msg_api(self, vid: str = '', page: int = 1):
        vids = ['', 'like', 'atmine', 'gift', 'sysmsg', 'resmsg']
        assert vid in vids
        self.req_count += 1
        param = {
            "quickViewId": 'upCollageMain',
            "reqID": self.req_count,
            "ajaxpipe": 1,
            "pageNum": page,
            "t": str(time.time_ns())[:13],
        }
        api_req = self.acer.client.get(apis['message'] + vid, params=param)
        if api_req.text.endswith("/*<!-- fetch-stream -->*/"):
            api_data = json.loads(api_req.text[:-25])
            page_obj = Bs(api_data.get('html', ''), 'lxml')
        else:
            return None
        item_data = list()
        total = str(page_obj.select_one('#listview').attrs['totalcount'])
        for item in page_obj.select('#listview > ul,.main-block-msg-item'):
            if vid == '':
                main_url = scheme + ':' + item.select_one('.intro').a.attrs['href']
                reply_url = item.select_one('a.msg-reply').attrs['href']
                item_data.append({
                    'content_url': main_url,
                    'content_title': item.select_one('.intro').a.text.strip(),
                    'replied': item.select_one('.msg-replied .inner').text.strip().replace('\xa0', ' '),
                    'uid': item.select_one('.titlebar .name').attrs['href'][17:],
                    'username': item.select_one('.titlebar .name').text,
                    'create_at': item.select_one('.titlebar .time').text.strip(),
                    'ncid': reply_url.split('#ncid=')[1],
                    'content': item.select_one('.msg-reply .inner').text.strip().replace('\xa0', ' '),
                    'intro': item.select_one('.content .intro').text.strip(),
                })
            elif vid == 'like':
                this_url = item.select_one('a.replied').attrs['href'].split('#')
                main_url = scheme + ':' + this_url[0]
                item_data.append({
                    'content_url': main_url,
                    'replied': item.select_one('.clamp-text .inner').text.strip().replace('\xa0', ' '),
                    'uid': item.select_one('.titlebar .name').attrs['href'][17:],
                    'username': item.select_one('.titlebar .name').text,
                    'ncid': this_url[1][5:],
                    'create_at': item.select_one('.titlebar span.time').text.strip(),
                    'intro': item.select_one('.titlebar').text.strip(),
                })
            elif vid == 'atmine':
                this_url = item.select_one('.content .msg-text').attrs['href']
                item_data.append({
                    'content_url': scheme + ':' + this_url.split('#')[0],
                    'ncid': this_url.split('#ncid=')[1],
                    'uid': item.select_one('.avatar-section').attrs['href'][17:],
                    'username': item.select_one('.titlebar-container .name').text,
                    'create_at': item.select_one('.titlebar-container span.time').text.strip(),
                    'intro': item.select_one('.titlebar-container .intro').text.strip(),
                })
            elif vid == 'gift':
                if 'moment-gift' in item.attrs['class']:
                    this_url = item.select_one('.msg-content').a.attrs['href']
                    intro = thin_string(item.select_one('.msg-content').text)
                    item_data.append({
                        'classify': 'moment',
                        'content_url': scheme + ':' + this_url,
                        'content_title': '动态',
                        'uid': item.select_one('.avatar-section').attrs['href'][17:],
                        'username': item.select_one('.content .name').text,
                        'create_at': item.select_one('.content span.time').text.strip(),
                        'intro': intro,
                        'banana': int(re.findall(r'(\d)根香蕉', intro)[0])
                    })
                else:
                    acer_url = item.select_one('p a:nth-of-type(1)')
                    this_url = item.select_one('p a:nth-of-type(2)')
                    intro = thin_string(item.select_one('p').text)
                    item_data.append({
                        'classify': 'content',
                        'content_url': scheme + ':' + this_url.attrs['href'],
                        'content_title': this_url.text,
                        'uid': acer_url.attrs['href'][17:],
                        'username': acer_url.text,
                        'create_at': item.select_one('.msg-item-time').text.strip(),
                        'intro': intro,
                        'banana': int(re.findall(r'(\d)根香蕉', intro)[0])
                    })
            elif vid == 'sysmsg':
                this_title = item.select_one('div:nth-of-type(1)').text.strip()
                this_content = item.select_one('div:nth-of-type(2)').get_text("|", strip=True)
                this_content = "\n".join([text for text in this_content.split("|") if not text.startswith('http')])
                this_content = re.sub('(>{2,})', '', this_content)
                this_link = item.select_one('div:nth-of-type(2)').a.attrs['href']
                item_data.append({
                    'content_url': this_link,
                    'content_title': this_title,
                    'create_at': item.select_one('.msg-item-time').text.strip(),
                    'intro': thin_string(this_content),
                })
            elif vid == 'resmsg':
                intro = item.select_one('p:nth-of-type(1)').text.strip()
                links = dict()
                for link in item.select('a'):
                    url_str = link.attrs['href']
                    if not url_str.startswith('http'):
                        url_str = scheme + ':' + url_str
                    for link_name in ['video', 'article', 'album', 'bangumi', 'up', 'live']:
                        if url_str.startswith(routes[link_name]) and link_name not in links:
                            links[link_name] = [link.text, url_str]
                item_data.append({
                    'create_at': item.select_one('p.msg-item-time').text.strip(),
                    'intro': thin_string(intro, True),
                    **links
                })
        return item_data, total

    def reply(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('', page)
        if obj is True:
            return [ReplyMsg(self, **i) for i in api_data]
        return api_data

    def like(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('like', page)
        if obj is True:
            return [LikeMsg(self, **i) for i in api_data]
        return api_data

    def at(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('atmine', page)
        if obj is True:
            return [AtMsg(self, **i) for i in api_data]
        return api_data

    def gift(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('gift', page)
        if obj is True:
            return [GiftMsg(self, **i) for i in api_data]
        return api_data

    def notice(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('sysmsg', page)
        if obj is True:
            return [NoticeMsg(self, **i) for i in api_data]
        return api_data

    def system(self, page: int = 1, obj: bool = False):
        api_data, total = self._get_msg_api('resmsg', page)
        if obj is True:
            return [SystemMsg(self, **i) for i in api_data]
        return api_data



# coding=utf-8
import json
import time
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import routes, apis

__author__ = 'dolacmeo'


class AcUp:
    resource_type = 5
    uid = None
    up_data = None
    up_page = None

    video_count = None
    article_count = None
    album_count = None
    following_count = None
    followed_count = None
    is_404 = False

    def __init__(self, acer, uid: [str, int], up_data: [dict, None] = None):
        self.uid = int(uid)
        self.up_data = dict() if up_data is None else up_data
        self.acer = acer
        self._get_acup()

    def _get_acup(self):
        api_req = self.acer.client.get(apis['userInfo'], params={"userId": self.uid})
        if api_req.json().get('result') != 0:
            self.is_404 = True
            return None
        self.up_data.update(api_req.json().get('profile', {}))

    @property
    def name(self):
        return self.up_data.get('userName', self.up_data.get('name'))

    def __repr__(self):
        return f"Acer([{self.uid}] @{self.name})".encode(errors='replace').decode()

    @property
    def referer(self):
        return f"{routes['up']}{self.uid}"

    def loading(self):
        if self.is_404 is True:
            return None
        page_req = self.acer.client.get(routes['up'] + self.uid)
        self.up_page = Bs(page_req.text, "lxml")
        self.video_count = self.up_page.select_one(
            '.ac-space-contribute-list > .tags > li[data-index=video]').attrs['data-count']
        self.article_count = self.up_page.select_one(
            '.ac-space-contribute-list > .tags > li[data-index=article]').attrs['data-count']
        self.album_count = self.up_page.select_one(
            '.ac-space-contribute-list > .tags > li[data-index=album]').attrs['data-count']
        self.following_count = self.up_page.select_one(
            '.tab-list > li[data-index=following] > span').text
        self.followed_count = self.up_page.select_one(
            '.tab-list > li[data-index=followed] > span').text

    def AcLive(self):
        return self.acer.acfun.AcLiveUp(self.uid)

    def follow_add(self, attention: [bool, None] = None):
        return self.acer.follow_add(self.uid, attention)

    def follow_remove(self):
        return self.acer.follow_remove(self.uid)

    def _get_data(self, viewer, page, limit, orderby):
        viewers = ['video', 'article', 'album', 'following', 'followed']
        assert viewer in viewers
        orders = ['newest', 'hotest']
        assert orderby in orders
        param = {
            "reqID": viewers.index(viewer) + 1,
            "ajaxpipe": 1,
            "type": viewer,
            "page": page,
            "pageSize": limit,
            "t": str(time.time_ns())[:13]
        }
        if viewers.index(viewer) < 3:
            param.update({"quickViewId": f"ac-space-{viewer}-list", "order": orderby})
        else:
            param.update({"quickViewId": f"ac-space-{viewer}-user-list"})
        req = self.acer.client.get(routes['up'] + self.uid, params=param)
        assert req.text.endswith("/*<!-- fetch-stream -->*/")
        return json.loads(req.text[:-25])

    def video(self, page=1, limit=10, orderby='newest'):
        data = list()
        acer_data = self._get_data('video', page, limit, orderby)
        for item in Bs(acer_data.get('html', ''), 'lxml').select('a.ac-space-video'):
            ac_num = item.attrs['href'][5:]
            infos = {
                'title': item.select_one('p.title').attrs['title'],
                'dougaId': ac_num,
                'coverUrl': item.select_one('.video > img').attrs['src'],
                'createTime': item.select_one('p.date').text.replace('/', '-'),
                'user': self.up_data
            }
            data.append(self.acer.acfun.AcVideo(ac_num, infos))
        return data

    def article(self, page=1, limit=10, orderby='newest'):
        data = list()
        acer_data = self._get_data('article', page, limit, orderby)
        for item in Bs(acer_data.get('html', ''), 'lxml').select('.ac-space-article'):
            ac_num = item.a.attrs['href'][5:]
            infos = {
                'title': item.a.attrs['title'],
                'dougaId': ac_num,
                'user': self.up_data
            }
            data.append(self.acer.acfun.AcArticle(ac_num, infos))
        return data

    def album(self, page=1, limit=10, orderby='newest'):
        data = list()
        acer_data = self._get_data('album', page, limit, orderby)
        for item in Bs(acer_data.get('html', ''), 'lxml').select('.ac-space-album'):
            ac_num = item.a.attrs['href'][5:]
            data.append(self.acer.acfun.AcAlbum(ac_num))
        return data

    def _follow(self, key, page=1, limit=10, orderby='newest'):
        assert key in ['following', 'followed']
        data = list()
        acer_data = self._get_data(key, page, limit, orderby)
        for item in Bs(acer_data.get('html', ''), 'lxml').select('li'):
            infos = {
                "id": item.select_one('div:nth-of-type(2) > a.name').attrs['href'][3:],
                "name": item.select_one('div:nth-of-type(2) > a.name').text
            }
            data.append(AcUp(self.acer, infos['id']))
        return data

    def following(self, page=1, limit=10, orderby='newest'):
        return self._follow("following", page, limit, orderby)

    def followed(self, page=1, limit=10, orderby='newest'):
        return self._follow("followed", page, limit, orderby)

    def report(self, crime: str, proof: str, description: str):
        return self.acer.acfun.AcReport.submit(
            self.referer, self.uid, self.resource_type,
            self.uid,
            crime, proof, description)



# coding=utf-8
from typing import Literal
from urllib import parse
from acfunsdk.source import routes, apis

__author__ = 'dolacmeo'


class AcRank:
    date_ranges = ['DAY', 'THREE_DAYS', 'WEEK']
    rank_data = None

    def __init__(self, acer,
                 cid: [int, None] = None,
                 sub_cid: [int, None] = None,
                 limit: int = 50,
                 date_range: (Literal['DAY', 'THREE_DAYS', 'WEEK'], None) = None):
        self.acer = acer
        self.cid = cid if isinstance(cid, int) else ""
        self.sub_cid = sub_cid if isinstance(sub_cid, int) else ""
        self.limit = limit
        self.date_range = date_range if date_range in self.date_ranges else self.date_ranges[0]
        self.get_data()

    def __repr__(self):
        this_id = self.sub_cid or self.cid or "ALL"
        return f"AcRank(#{this_id} {self.date_range})"

    @property
    def referer(self):
        param = {
            "pcid": self.cid or "-1",
            "cid": self.sub_cid or "-1",
            "range": self.date_range
        }
        return f"{routes['rank']}?{parse.urlencode(param)}"

    def get_data(self):
        param = {
            "channelId": self.cid,
            "subChannelId": self.sub_cid,
            "rankLimit": self.limit,
            "rankPeriod": self.date_range
        }
        api_req = self.acer.client.get(apis['rank_list'], params=param)
        if api_req.json().get('result') == 0:
            self.rank_data = api_req.json().get('rankList')

    def channel(self):
        return self.acer.acfun.AcChannel(self.sub_cid or self.cid)

    def contents(self):
        if self.rank_data is None:
            return None
        data_list = list()
        for content in self.rank_data:
            if content['contentType'] == 2:
                data_list.append(self.acer.acfun.AcVideo(content['dougaId'], content))
            elif content['contentType'] == 3:
                data_list.append(self.acer.acfun.AcArticle(content['resourceId'], content))
        return data_list

    def ups(self):
        if self.rank_data is None:
            return None
        data_list = list()
        for content in self.rank_data:
            data_list.append(self.acer.acfun.AcUp(content['userId']))
        return data_list



# coding=utf-8
import json
import time
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import routes, apis, channel_data
from acfunsdk.page.utils import match1

__author__ = 'dolacmeo'


class BlockContent:
    content_data = None

    def __init__(self, acer, content_data: dict):
        self.acer = acer
        self.content_data = content_data

    def __repr__(self):
        if self.name:
            key = f"[{self.keyname}]" if self.keyname else ""
            return f"AcContent({self.name}{key})"
        return f"AcContent(#{self.blockId} Σ{self.contentCount})"

    @property
    def blockId(self):
        return self.content_data.get('blockId')

    @property
    def name(self):
        return self.content_data.get('name')

    @property
    def keyname(self):
        return self.content_data.get('interfaceParameter', '').strip()

    @property
    def contentCount(self):
        return self.content_data.get('contentCount')

    def list(self):
        data_list = list()
        for v in self.content_data.get('webContents'):
            if v['mediaType'] == 0:
                data_list.append(self.acer.acfun.AcVideo(v['mediaId']))
            elif v['mediaType'] == 1:
                data_list.append(self.acer.acfun.AcArticle(v['mediaId']))
            elif v['mediaType'] == 2:
                data_list.append(self.acer.acfun.AcBangumi(v['mediaId']))
            elif v['mediaType'] == 4:
                data_list.append(self.acer.acfun.AcUp(v))
            elif v['mediaType'] == 8:
                data_list.append(self.acer.acfun.AcImage(v['image'], v['link'], v['title']))
        return data_list


class ChannelBlock:
    block_data = None

    def __init__(self, acer, block_data: dict):
        self.acer = acer
        self.block_data = block_data

    def __repr__(self):
        return f"AcBlock(#{self.blockType} {self.name})"

    @property
    def name(self):
        return self.block_data.get('name', '').strip()

    @property
    def blockType(self):
        return self.block_data.get('blockType')

    def contents(self):
        return [BlockContent(self.acer, content) for content in self.block_data.get('content', [])]


class AcChannel:
    channel_data = channel_data
    nav_data = dict()
    channel_obj = None
    is_main = False
    parent_data = None
    sub_data = None
    is_404 = False

    def __init__(self, acer, cid, nav_info: dict):
        self.acer = acer
        self.cid = str(cid)
        self.info = nav_info
        self._get_channel_info()

    @property
    def ctype(self):
        if self.is_404:
            return None
        if self.is_main:
            if self.cid != '63':
                return 'main'
            return 'wen'
        if self.parent_data['channelId'] == '63':
            return 'wen'
        return 'videos'

    @property
    def referer(self):
        return f"{routes['index']}/v/list{self.cid}/index.htm"

    @property
    def _main_channels(self):
        return {x["channelId"]: x for x in self.channel_data}

    def _get_channel_info(self):
        for channel in self.channel_data:
            if self.cid == channel['channelId']:
                self.parent_data = channel
                self.info = self.parent_data
                break
            for sub in channel['children']:
                if self.cid == sub['channelId']:
                    self.parent_data = channel
                    self.sub_data = sub
                    self.info = self.sub_data
                    break
        self.is_main = self.sub_data is None
        if self.parent_data is None:
            self.is_404 = True

    def __repr__(self):
        if self.is_404 is True:
            return f"AcChannel(#{self.cid} 404)"
        return f"AcChannel(#{self.cid} {self.info['name']})"

    def loading(self):
        if not self.is_main:
            print("Is sub channels, just use videos.")
            return False
        page_req = self.acer.client.get(self.referer)
        self.channel_obj = Bs(page_req.text, 'lxml')
        json_text = match1(page_req.text, r"(?s)__INITIAL_STATE__\s*=\s*(\{.*?\});")
        if json_text is None:
            self.is_404 = True
            return False
        self.channel_data = json.loads(json_text)

    def hot_words(self):
        if not self.is_main:
            return None
        if self.channel_data is None:
            self.loading()
        return self.channel_data['channel']['hotWordList']

    def blocks(self):
        if not self.is_main:
            return None
        if self.channel_data is None:
            self.loading()
        if self.cid == '63':
            return [ChannelBlock(self.acer, data) for data in self.channel_data['article']['blockList']]
        return [ChannelBlock(self.acer, data) for data in self.channel_data['channel']['blockList']]

    def ranks(self, limit: int = 50, date_range: str = None):
        if self.is_main:
            cid = int(self.cid)
            sub_cid = None
        else:
            cid = int(self.parent_data['channelId'])
            sub_cid = int(self.cid)
        return self.acer.acfun.AcRank(cid, sub_cid, limit, date_range=date_range)

    def articles(self):
        if self.ctype != 'wen':
            return None
        if self.cid == '63':
            return self.acer.acfun.AcWen()
        rids = list()
        for item in self.info.get("realms", []):
            rids.append(item['realmId'])
        return self.acer.acfun.AcWen(rids)

    def videos(self,
               page: int = 1,
               sortby: [str, None] = None,
               duration: [str, None] = None,
               datein: [str, None] = None,
               obj: bool = True):
        if self.ctype != 'videos':
            return None
        sortby_list = {
            "rankScore": "综合",
            "createTime": "最新投稿",
            "viewCount": "播放最多",
            "commentCount": "评论最多",
            "bananaCount": "投蕉最多",
            "danmakuCount": "弹幕最多"
        }
        assert sortby in sortby_list or sortby is None
        duration_list = {
            "all": "全部",
            "0,5": "5分钟以下",
            "5,30": "5-30分钟",
            "30,60": "30-60分钟",
            "60,": "60分钟以上"
        }
        assert duration in duration_list or duration is None
        datein_list = {
            "default": "近三个月",
            "20200101,20210101": "2020",
            "20190101,20200101": "2019",
            "20180101,20190101": "2018",
            "20170101,20180101": "2017",
            "20160101,20170101": "2016",
            "20150101,20160101": "2015",
            "20100101,20150101": "2014-2010",
            ",20100101": "更早",
        }
        assert datein in datein_list or datein is None
        api_req = self.acer.client.get(self.referer, params={
            "sortField": "rankScore" if sortby is None else sortby,
            "duration": "all" if duration is None else duration,
            "date": "default" if datein is None else datein,
            "page": page,
            "quickViewId": "listwrapper",
            "reqID": 0,
            "ajaxpipe": 1,
            "t": str(time.time_ns())[:13]
        })
        assert api_req.text.endswith("/*<!-- fetch-stream -->*/")
        api_data = json.loads(api_req.text[:-25])
        api_obj = Bs(api_data.get('html', ''), 'lxml')
        v_datas = list()
        for item in api_obj.select('.list-content-item'):
            item_data = {
                'ac_num': item.select_one('.list-content-top').attrs['href'][5:],
                'title': item.select_one('.list-content-title').a.attrs['title'],
                'duration': item.select_one('.danmaku-mask .video-time').text,
                'commentCountShow': item.select_one('.list-content-data .commentCount').text,
                'viewCountShow': item.select_one('.list-content-data .viewCount').text,
                'coverUrl': item.select_one('.list-content-cover').attrs['src'],
                'user': {
                    'id': item.select_one('.list-content-uplink').attrs['data-uid'],
                    'name': item.select_one('.list-content-uplink').attrs['title'],
                }
            }
            if obj is True:
                v_datas.append(self.acer.acfun.AcVideo(item_data['ac_num'], item_data))
            else:
                v_datas.append(item_data)
        return v_datas



# coding=utf-8
import time
import json
import math
from urllib import parse
from bs4 import BeautifulSoup as Bs
from acfunsdk.source import routes, apis
from acfunsdk.page.utils import get_channel_info, get_page_pagelets, match1

__author__ = 'dolacmeo'


class AcBangumi:
    resource_type = 1
    aa_num = None
    page_obj = None
    page_pagelets = []
    bangumi_data = None
    bangumi_list = None
    vid = None
    item_id = None
    resourceType = 6
    is_404 = False

    def __init__(self, acer, aa_num: [str, int]):
        if isinstance(aa_num, str) and aa_num.startswith('aa'):
            aa_num = aa_num[2:]
        self.aa_num = str(aa_num)
        self.acer = acer
        self.loading()
        if self.is_404 is False:
            self.set_video()

    def __repr__(self):
        if self.is_404:
            return f"AcBangumi([ac{self.aa_num}]咦？世界线变动了。看看其他内容吧~)"
        title = self.bangumi_data.get('showTitle', self.bangumi_data.get('bangumiTitle', ""))
        return f"AcBangumi([ac{self.aa_num}_{self.item_id}]{title})".encode(errors='replace').decode()

    @property
    def referer(self):
        if self.item_id is None:
            return f"{routes['bangumi']}{self.aa_num}"
        return f"{routes['bangumi']}{self.aa_num}_36188_{self.item_id}"

    def loading(self):
        req = self.acer.client.get(routes['bangumi'] + self.aa_num)
        self.is_404 = req.status_code // 100 != 2
        if self.is_404:
            return False
        self.page_obj = Bs(req.text, 'lxml')
        script_bangumidata = match1(req.text, r"(?s)bangumiData\s*=\s*(\{.*?\});")
        self.bangumi_data = json.loads(script_bangumidata)
        script_bangumilist = match1(req.text, r"(?s)bangumiList\s*=\s*(\{.*?\});")
        self.bangumi_list = json.loads(script_bangumilist)
        self.bangumi_data.update(get_channel_info(req.text))
        self.item_id = self.bangumi_data.get("itemId")
        self.vid = self.bangumi_data.get("videoId")
        self.page_pagelets = get_page_pagelets(self.page_obj)

    @property
    def season_data(self):
        return self.bangumi_data.get('relatedBangumis', [])

    @property
    def episode_data(self):
        return self.bangumi_list.get('items', [])

    def set_video(self, num=1):
        if num > len(self.episode_data):
            return False
        this_episode = self.episode_data[num - 1]
        self.vid = this_episode['videoId']
        self.item_id = this_episode['itemId']
        self.bangumi_data.update({
            'videoId': self.vid, 'itemId': self.item_id,
            'showTitle': f"{this_episode['bangumiTitle']} {this_episode['episodeName']} {this_episode['title']}"
        })
        return True

    def danmaku(self):
        return self.acer.acfun.AcDanmaku(self.bangumi_data)

    def comment(self):
        return self.acer.acfun.AcComment(f"{self.aa_num}_{self.vid}", 6)

    def like(self):
        return self.acer.like_add(self.item_id, 18)

    def like_cancel(self):
        return self.acer.like_delete(self.item_id, 18)

    def favorite_add(self):
        return self.acer.favourite.add(self.aa_num, 1)

    def favorite_cancel(self):
        return self.acer.favourite.cancel(self.aa_num, 1)

    def banana(self):
        return self.acer.throw_banana(self.item_id, 18, 1)

    def report(self, crime: str, proof: str, description: str):
        return self.acer.acfun.AcReport.submit(
            self.referer, self.aa_num, self.resource_type, "0",
            crime, proof, description)


class AcBangumiList:
    page_obj = None
    menu_filter = list()
    default_filter = None

    def __init__(self, acer):
        self.acer = acer
        self.loading()

    def __repr__(self):
        return f"AcBangumiList(番剧列表 - AcFun弹幕视频网)"

    @property
    def referer(self):
        return f"{routes['bangumi_list']}"

    def loading(self):
        index_req = self.acer.client.get(routes['bangumi_list'])
        self.page_obj = Bs(index_req.text, 'lxml')
        firsts = []
        for menu in self.page_obj.select(".ac-menu .ac-menu-filter"):
            menu_id = int(menu.attrs['data-id'])
            filter_title = menu.select_one('.ac-menu-filter-title').text.replace(":", "")
            filter_item = {item.attrs['data-id']: item.text for item in menu.select(".ac-menu-filter-item")}
            self.menu_filter.append({'title': filter_title, 'item': filter_item, 'menu_id': menu_id})
            default_id = menu.select_one(".ac-menu-filter-item").attrs['data-id']
            firsts.append([menu_id, default_id])
        self.menu_filter = sorted(self.menu_filter, key=lambda m: m['menu_id'])
        firsts = sorted(firsts, key=lambda i: i[0])
        self.default_filter = ",".join([n[1] for n in firsts])

    def _filter_check(self, filters: [str, list]):
        filters = filters.split(",") if isinstance(filters, str) else filters
        if len(filters) != len(self.menu_filter):
            return False
        for n in range(len(filters)):
            if filters[n] not in self.menu_filter[n]['item']:
                print(f"{filters[n]} not in {self.menu_filter[n]['title']}: {self.menu_filter[n]['item'].keys()}")
                return False
        return True

    def tans_filters(self, words: [str, list]):
        filters = words.split(",") if isinstance(words, str) else words
        if len(filters) != len(self.menu_filter):
            return False
        v = list()
        for n in range(len(filters)):
            if filters[n] not in self.menu_filter[n]['item'].values():
                return False
            _inverse = {v: k for k, v in self.menu_filter[n]['item'].items()}
            v.append(_inverse[filters[n]])
        return ",".join(v)

    def page_from_url(self, full_url: str, obj: bool = False):
        filters = self.default_filter
        page = 1
        param = parse.urlsplit(full_url).query
        param = parse.parse_qs(param)
        if "filters" in param:
            filters = param['filters'][0]
        if "pageNum" in param:
            page = int(param['pageNum'][0])
        return self.page(filters, page, obj)

    def page(self, filters: [str, list, None] = None, page: int = 1, obj: bool = False):
        filters = self.default_filter if filters is None else filters
        if self._filter_check(filters) is False:
            return None
        filters = ",".join(filters) if isinstance(filters, list) else filters
        param = {
            "quickViewId": "bangumiList",
            "filters": filters,
            "reqID": 1, "ajaxpipe": 1,
            "pageNum": page,
            "t": str(time.time_ns())[:13]
        }
        page_req = self.acer.client.get(routes['bangumi_list'], params=param)
        if not page_req.text.endswith("/*<!-- fetch-stream -->*/"):
            return None
        api_data = json.loads(page_req.text[:-25])
        page_obj = Bs(api_data.get('html', ''), 'lxml')
        ac_list = page_obj.select_one(".ac-mod-list")
        item_total = int(ac_list.attrs['data-totalcount'])
        page_size = int(ac_list.attrs['data-pagesize'])
        max_page = math.ceil(item_total / page_size)
        items = list()
        for item in ac_list.select("li.ac-mod-li"):
            item_data = {
                "aa_num": item.attrs['data-aid'],
                "link": item.select_one('.ac-mod-link').attrs['href'],
                "cover": item.select_one('img.ac-mod-cover').attrs['src'],
                "cover_stats": item.select_one('.ac-mod-cover-stats').text,
                "need_pay": item.select_one('.pay-toast') is not None,
                "title": item.select_one('.ac-mod-title').text,
                "latestItem": item.select_one('.ac-mod-title > em').text,
            }
            if obj is True:
                items.append(AcBangumi(self.acer, item_data['aa_num']))
            else:
                items.append(item_data)
        return {
            "items": items,
            "page_now": page,
            "page_total": max_page,
            "page_size": page_size,
            "total_count": item_total
        }

    pass



# coding=utf-8
import os
import time
import json
import subprocess
from acfunsdk.source import scheme, domains, routes, apis

__author__ = 'dolacmeo'


class AcLive:
    last_update = None
    raw_data = dict()

    def __init__(self, acer):
        self.acer = acer
        self._get_list()

    @property
    def referer(self):
        return f"{routes['live_index']}"

    def __repr__(self):
        return f"AcLive(直播 - AcFun弹幕视频网)"

    def _get_list(self):
        if self.last_update is None or time.time() - self.last_update >= 60:
            api_req = self.acer.client.get(apis['live_list'])
            self.raw_data = api_req.json()
            self.last_update = time.time()

    def list(self):
        self._get_list()
        lives = list()
        for x in self.raw_data.get('liveList', []):
            lives.append(AcLiveUp(self.acer, x.get('authorId'), x))
        return lives

    def get(self, uid: [int, str]):
        return AcLiveUp(self.acer, uid)


class AcLiveUp:
    uid = None
    raw = None
    acws = None
    is_open = False
    media_data = None
    is_404 = False
    liveId = None
    enterRoomAttach = None
    availableTickets = []
    report_data = dict()

    def __init__(self, acer, uid: [int, str], raw: [dict, None] = None):
        self.acer = acer
        self.uid = uid
        self.raw = raw
        if self.raw is None:
            self.infos()
        self.AcUp = self.acer.acfun.AcUp(self.uid)
        self.media_data = self.media_list()
        self.is_404 = self.AcUp.is_404
        self.report_data['host'] = self._get_report_data('liveStream')
        self.report_data['audience'] = self._get_report_data('liveStreamAudience')

    @property
    def title(self):
        return self.raw.get('title', '')

    @property
    def username(self):
        return self.raw.get('user', {}).get('name', '')

    def up(self):
        return self.acer.acfun.AcUp(self.uid)

    @property
    def referer(self):
        return f"{routes['live']}{self.uid}"

    @property
    def past_time(self):
        p = (time.time_ns() // pow(10, 6)) - self.raw.get("createTime", 0)
        return p if p > 0 else -1

    def __repr__(self):
        return f"AcLive(#{self.uid} {self.title} @{self.username})".encode(errors='replace').decode()

    def infos(self, key: [str, None] = None):
        param = {"authorId": self.uid}
        api_req = self.acer.client.get(apis['live_info'], params=param)
        self.raw = api_req.json()
        if key is not None:
            return self.raw.get(key)
        return self.raw

    def contents(self):
        param = {"authorId": self.uid}
        api_req = self.acer.client.get(apis['live_up_contents'], params=param)
        api_data = api_req.json()
        return [self.acer.acfun.AcVideo(i.get('dougaId')) for i in api_data.get('contributeList', {}).get('feed', [])]

    def _api_action(self, api_name: str, form_data: dict):
        param = {
            "subBiz": "mainApp",
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "userId": self.acer.uid,
            "did": self.acer.client.cookies.get('_did'),
        }
        param = self.acer.update_token(param)
        api_req = self.acer.client.post(apis[api_name], params=param, data=form_data,
                                        headers={'referer': f"{scheme}://{domains['live']}/"})
        return api_req.json()

    def my_balance(self):
        api_data = self._api_action('live_balance', {"visitorId": self.uid})
        values = api_data.get("data", {}).get("payWalletTypeToBalance", {})
        return {"acb": values.get('1', 0), "banana": values.get('2', 0)}

    def media_list(self):
        form_data = {"authorId": self.uid, "pullStreamType": "FLV"}
        api_data = self._api_action('live_play', form_data)
        if api_data.get('result') == 129004:
            self.is_open = False
            return False
        elif api_data.get('result') != 1:
            return False
        self.is_open = True
        self.liveId = api_data['data'].get("liveId")
        self.availableTickets = api_data['data'].get("availableTickets", [])
        self.enterRoomAttach = api_data['data'].get("enterRoomAttach")
        api_data = api_data.get('data', {}).get('videoPlayRes', "")
        live_data = json.loads(api_data).get('liveAdaptiveManifest', [])[0]
        live_adapt = live_data.get('adaptationSet', {}).get('representation', {})
        return live_adapt

    def medal_info(self):
        api_req = self.acer.client.post(apis['live_medal'], params={"uperId": self.uid})
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def push_danmaku(self, content: str):
        if self.is_open is False:
            return False
        form_data = {
            "visitorId": self.uid,
            "liveId": self.raw.get("liveId"),
            "content": content
        }
        api_data = self._api_action('live_danmaku', form_data)
        return api_data.get('result') == 1

    def like(self, times: int, max_retry: int = 10):
        if self.is_open is False:
            return False
        form_data = {
            "visitorId": self.uid,
            "liveId": self.raw.get("liveId"),
            "count": 1,
            "durationMs": 800
        }
        retry = 0
        while times > 0 and retry <= max_retry:
            api_data = self._api_action('live_like', form_data)
            if api_data.get('result') == 1 and api_data.get('data', {}).get('count') == 1:
                times -= 1
            else:
                retry += 1
            time.sleep(1)
        return True

    def gift_list(self):
        if self.is_open is False:
            return False
        form_data = {
            "visitorId": self.uid,
            "liveId": self.raw.get("liveId"),
        }
        api_data = self._api_action('live_gift_list', form_data)
        gifts = dict()
        for gift in api_data.get('data', {}).get('giftList', []):
            gifts.update({f"{gift['giftId']}": gift})
        ext_gift = api_data.get('data', {}).get('externalDisplayGift', {}).get('giftList', [])
        if len(ext_gift):
            ext_gift = ext_gift[0]
            ext_gift.update({"tipsDelayTime": api_data.get('data', {}).get('externalDisplayGiftTipsDelayTime', 0)})
            gifts.update({f"{ext_gift['giftId']}": ext_gift})
        return gifts

    def send_gift(self, gift_id: int, size: int, times: int = 1, max_retry: int = 10):
        if self.is_open is False:
            return False
        gift_data = self.gift_list()
        # 判断礼物类型是否存在
        if f"{gift_id}" not in gift_data:
            return False
        this_gift = gift_data.get(f"{gift_id}", {})
        # 判断礼物数量是否正确
        if size not in this_gift.get("allowBatchSendSizeList", []):
            return False
        # 判断余额是否充足
        gift_type = this_gift.get("payWalletType", 0)
        gift_type_name = [0, 'acb', 'banana'][gift_type]
        gift_price = this_gift.get("giftPrice")
        total_price = gift_price * size * times
        balance = self.my_balance()
        if balance[gift_type_name] < total_price:
            return False
        combo_id = f"{gift_id}_{gift_id}_{time.time():.0f}000"
        form_data = {
            "visitorId": self.uid,
            "liveId": self.raw.get("liveId"),
            "giftId": gift_id,
            "batchSize": size,
            "comboKey": combo_id
        }
        retry = 0
        while times > 0 and retry <= max_retry:
            api_data = self._api_action('live_send_gift', form_data)
            if api_data.get('result') == 1:
                times -= 1
            else:
                retry += 1
            time.sleep(0.1)
        return True

    def playing(self, potplayer: [str, None] = None, quality: int = 1):
        live_adapt = self.media_list()
        if live_adapt is False:
            return False
        if quality not in range(len(live_adapt)):
            quality = 1
        create_time = self.raw.get('createTime', 0) // 1000
        start_time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(create_time))
        live_title = " ".join([str(self), live_adapt[quality]['name'], f"🎬 {start_time}"])
        live_obs_stream = live_adapt[quality]['url']
        if potplayer is not None and os.path.exists(potplayer):
            print(f"[{potplayer}] 开始播放......\r\n {live_title}")
            subprocess.Popen([potplayer, live_obs_stream, "/title", f'"{live_title}"'], stdout=subprocess.PIPE)
        else:
            print(f"未设置PotPlayer 请使用串流地址 请自行播放 \r\n {live_obs_stream}")
        return live_obs_stream

    def _get_report_data(self, sub_biz: str):
        assert sub_biz in ['liveStream', 'liveStreamAudience']
        api_name = {
            'liveStream': "live_report_h0",
            'liveStreamAudience': "live_report_a0"
        }
        data = {
            "subBiz": sub_biz,
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "appver": "1.0.0",
            "userId": self.acer.uid,
            "did": self.acer.did,
        }
        data = self.acer.update_token(data)
        api_req = self.acer.client.post(apis[api_name[sub_biz]], params=data, json=data)
        api_data = api_req.json()
        assert api_data.get("result") == 1
        config = json.loads(api_data.get('json'))
        options = list()
        for x in config['config']['props']['children']:
            text = x['label']
            key = x['output']
            option_id = api_data['reportCodeMap'][key]
            options.append({'id': option_id, 'key': key, 'text': text})
        return options

    def _report_req(self, sub_biz: str, code: int, reason: [str, None] = None):
        api_name = {
            'liveStream': "live_report_h1",
            'liveStreamAudience': "live_report_a1"
        }
        data = {
            "reportCode": code,
            "reportedObject": f"{self.uid}",
            "extra": "{\"report_time\":" + str(self.past_time) + "}"
        }
        if isinstance(reason, str):
            data['reportReason'] = {"reason": reason}
        param = {
            "subBiz": sub_biz,
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "appver": "1.0.0",
            "userId": self.acer.uid,
            "did": self.acer.did,
        }
        api_req = self.acer.client.post(apis[api_name[sub_biz]], params=param, json=data)
        api_data = api_req.json()
        return api_data

    def report_host(self, code: int, reason: [str, None] = None):
        codes = [i['id'] for i in self.report_data['host']]
        assert code in codes
        return self._report_req('liveStream', code, reason)

    def report_audience(self, code: int, reason: [str, None] = None):
        codes = [i['id'] for i in self.report_data['audience']]
        assert code in codes
        return self._report_req('liveStreamAudience', code, reason)



# coding=utf-8
from functools import wraps

__all__ = (
    'need_login',
    'AcExploded',
    'NotInCar',
    'ShuiNi',
    'NiShuiA',
    'LuanJiangHua',
    'TingBuDong'
)


def need_login(f):
    @wraps(f)
    def wrapper(self, *args, **kwargs):
        if self.__class__.__name__ == 'Acer':
            logined = self.is_logined
        else:
            logined = self.acer.is_logined
        if logined is False:
            raise NotInCar("先登录啊！")
        return f(self, *args, **kwargs)
    return wrapper


class AcExploded(ConnectionError):
    """阿禅爆炸 (今天A站挂了吗？)"""


class NotInCar(PermissionError):
    """没上车 (尚未登录)"""


class ShuiNi(FileNotFoundError):
    """水逆 (404被删掉)"""


class NiShuiA(PermissionError):
    """你谁啊 (非本人无权操作)"""


class LuanJiangHua(ConnectionRefusedError):
    """乱讲话 (401被服务器拒绝)"""


class TingBuDong(ValueError):
    """听不懂 (数据格式已变更)"""



# coding=utf-8
import re
import httpx
from urllib import parse
from acfunsdk import source
from .index import AcIndex
from .channel import AcChannel
from .rank import AcRank
from .search import AcSearch
from .member import AcUp
from .article import AcArticle, AcWen
from .bangumi import AcBangumi, AcBangumiList
from .video import AcVideo
from .album import AcAlbum
from .moment import AcMoment, MyMoment
from .live import AcLive, AcLiveUp
from .doodle import AcDoodle
from .comment import AcComment
from .danmaku import AcDanmaku
from .message import MyMessage
from .acer import MyFansClub, MyFollow, MyFavourite, MyAlbum, MyContribute, MyDanmaku, BananaMall
from .extra import AcLink, AcImage, AcHelp, AcInfo, AcAcademy, AcReport, AcDownload
from .utils import B64s

__author__ = 'dolacmeo'


class AcFun:
    nav_data = dict()
    channel_data = source.channel_data
    AcInfo = AcInfo
    AcReport = AcReport
    AcAcademy = AcAcademy
    AcHelp = AcHelp
    resource_type_map = {
        "1": "AcBangumi",  # 番剧
        "2": "AcVideo",  # 视频稿件
        "3": "AcArticle",  # 文章稿件
        "4": "AcAlbum",  # 合辑
        "5": "AcUp",  # 用户
        "6": "AcComment",  # 评论
        # "8": "私信",
        "10": "AcMoment",  # 动态
    }
    routes_type_map = {
        "bangumi": "AcBangumi",  # 番剧
        "video": "AcVideo",  # 视频稿件
        "share": "AcVideo",  # 视频稿件
        "article": "AcArticle",  # 文章稿件
        "album": "AcAlbum",  # 合辑
        "up": "AcUp",  # 用户
        "live": "AcLiveUp",  # 用户直播
        "moment": "AcMoment",  # 用户动态
        "doodle": "AcDoodle",  # 涂鸦页面
    }

    def __init__(self, acer):
        self.acer = acer
        self.cdn_domain = self.acer.client.post(source.apis['cdn_domain'], headers={
            "referer": source.routes['index']}).json().get('domain')
        self._get_nav()

    def _get_nav(self):
        data = self.acer.client.get(source.apis['nav']).json().get("data", [])
        for i in data:
            if i['cid'] != 0:
                self.nav_data.update({str(i['cid']): {x: i[x] for x in i if x != 'children'}})
            for j in i['children']:
                if j['cid'] != 0:
                    self.nav_data.update({str(j['cid']): {y: j[y] for y in j if y != 'children'}})

    @property
    def referer(self):
        return f"{source.routes['index']}"

    def search_user(self, keyword: str, page: int = 1):
        api_req = self.acer.client.get(source.apis['search_user'], params={'keyword': keyword, "pCursor": page})
        return api_req.json()

    def username_check(self, name: str):
        api_req = self.acer.client.post(source.apis['check_username'], data={'name': name})
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return True
        print(api_data['error_msg'])
        return False

    def AcIndex(self):
        return AcIndex(self.acer)

    def AcChannel(self, cid):
        return AcChannel(self.acer, cid, self.nav_data.get(int(cid)))

    def AcBangumiList(self):
        return AcBangumiList(self.acer)

    def AcWen(self,
              realm_ids: [list, None] = None,
              sort_type: str = "createTime",
              time_range: str = "all",
              only_original: bool = False,
              limit: int = 10):
        return AcWen(self.acer, realm_ids, sort_type, time_range, only_original, limit)

    def AcRank(self,
               cid: [int, None] = None,
               sub_cid: [int, None] = None,
               limit: int = 50,
               date_range: [str, None] = None):
        return AcRank(self.acer, cid, sub_cid, limit, date_range)

    def AcSearch(self, keyword: [str, None] = None, s_type: [str, None] = None):
        return AcSearch(self.acer, keyword, s_type)

    def AcUp(self, uid):
        return AcUp(self.acer, uid)

    def AcLiveUp(self, uid, raw=None):
        return AcLiveUp(self.acer, uid, raw)

    def AcArticle(self, ac_num, data=None):
        return AcArticle(self.acer, ac_num, data)

    def AcVideo(self, ac_num, data=None):
        return AcVideo(self.acer, ac_num, data)

    def AcBangumi(self, aa_num):
        return AcBangumi(self.acer, aa_num)

    def AcAlbum(self, ac_num):
        return AcAlbum(self.acer, ac_num)

    def AcLive(self):
        return AcLive(self.acer)

    def AcDoodle(self, doodle_id: str):
        return AcDoodle(self.acer, doodle_id)

    def AcMoment(self, am_num):
        return AcMoment(self.acer, am_num)

    def AcDownload(self):
        return AcDownload(self.acer)

    def AcComment(self, rtype: int, rid: int):
        return AcComment(self.acer, rtype, rid)

    def AcDanmaku(self, video_data: dict):
        return AcDanmaku(self.acer, video_data)

    def AcImage(self, src, url=None, name=None):
        return AcImage(self.acer, src, url, name)

    def AcLink(self, url, title=None):
        return AcLink(self.acer, url, title)

    def resource(self, rtype: int, rid: int):
        assert str(rtype) in self.resource_type_map.keys()
        return getattr(self, self.resource_type_map[str(rtype)])(rid)

    def get(self, url_str: str, title=None):
        if url_str.startswith("http://"):
            url_str = url_str.replace("http://", "https://")
        if url_str in [source.routes['index'], source.routes['index'] + "/"]:
            return self.AcIndex()
        elif url_str in [source.routes['live_index'], source.routes['live_index'] + "/"]:
            return self.AcLive()
        for link_name in self.routes_type_map.keys():
            if url_str.startswith(source.routes[link_name]):
                ends = url_str[len(source.routes[link_name]):]
                return getattr(self, self.routes_type_map[link_name])(ends)
        for link_name in ['rank', 'bangumi_list']:
            if url_str.startswith(source.routes[link_name]):
                ends = url_str[len(source.routes[link_name]):]
                if link_name == 'bangumi_list':
                    return self.AcBangumiList()
                elif link_name == 'rank':
                    q = parse.parse_qs(parse.urlsplit(url_str).query)
                    kw = {
                        'cid': None if q.get('pcid', "-1") == "-1" else int(q['pcid'][0]),
                        'sub_cid': None if q.get('cid', "-1") == "-1" else int(q['cid'][0]),
                        'date_range': q.get("range", ['DAY'])[0],
                    }
                    return self.AcRank(**kw)
                return getattr(self, f"Ac{link_name.capitalize()}")(ends)
        channel_rex = re.compile(rf"^{source.routes['index']}/v/list(\d+)/index.htm").findall(url_str)
        if channel_rex:
            return self.AcChannel(channel_rex[0])
        if "//hd.acfun.cn/s/" in url_str:
            req3xx = httpx.get(url_str)
            req_redirect = parse.urlsplit(req3xx.headers.get("Location", ""))
            w_link = parse.parse_qs(req_redirect.query).get("wLink", [])
            if len(w_link):
                return self.get(w_link[0])
        if url_str.startswith('http') and parse.urlsplit(url_str).netloc.endswith('acfun.cn'):
            if parse.urlsplit(url_str).netloc in self.cdn_domain:
                return AcImage(self.acer, url_str)
            return AcLink(self.acer, url_str, title)
        return None



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['acfunsdk', 'acfunsdk.page']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4,<5', 'httpx>=0.23,<0.24', 'lxml>=4,<5']

setup_kwargs = {
    'name': 'acfunsdk',
    'version': '0.9.1',
    'description': 'acfunsdk - UNOFFICEICAL',
    'long_description': '# acfunsdk - **UNOFFICEICAL**\n\n<br />\n\n<p align="center">\n<a href="https://github.com/dolaCmeo/acfunsdk">\n<img height="100" src="https://s3.dualstack.us-east-2.amazonaws.com/pythondotorg-assets/media/files/python-logo-only.svg" alt="">\n<img height="100" src="https://ali-imgs.acfun.cn/kos/nlav10360/static/common/widget/header/img/acfunlogo.11a9841251f31e1a3316.svg" alt="">\n</a>\n</p>\n\n<br />\n\nacfunsdk是 **非官方的 [AcFun弹幕视频网][acfun.cn]** Python库。\n\n> 声明：`acfunsdk`是python的学习工具，并未破解任何acfun相关内容。代码完全公开，仅用于交流学习。\n> 如涉及版权等相关问题，请遵守acfun相关协议及法律法规。如有bug或其他疑问，欢迎发布[issues][Issue]。\n\n- - -\n\n**Python** : `Python>=3.7`， 本体请自行[下载安装][python]。\n\n## [从PyPI安装](https://pypi.org/project/acfunsdk/)\n\n```shell\npython -m pip install acfunsdk\n```\n\n- - -\n\n## 使用方法\n\n\n### 实例化获取对象\n```python\nfrom acfunsdk import Acer\n# 实例化一个Acer\nacer = Acer(debug=True)\n# 登录用户(成功登录后会自动保存 \'<用户名>.cookies\')\n# 请注意保存，防止被盗\nacer.login(username=\'you@email.com\', password=\'balalabalala\')\n# 读取用户(读取成功登录后保存的 \'<用户名>.cookies\')\nacer.loading(username=\'13800138000\')\n# 每日签到，领香蕉🍌\nacer.signin()\n# 通过链接直接获取内容对象\n# 目前支持 9种内容类型：\n# 视  频: https://www.acfun.cn/v/ac4741185\ndemo_video = acer.get("https://www.acfun.cn/v/ac4741185")\nprint(demo_video)\n# 文  章: https://www.acfun.cn/a/ac16695813\ndemo_article = acer.get("https://www.acfun.cn/a/ac16695813")\nprint(demo_article)\n# 合  集: https://www.acfun.cn/a/aa6001205\ndemo_album = acer.get("https://www.acfun.cn/a/aa6001205")\nprint(demo_album)\n# 番  剧: https://www.acfun.cn/bangumi/aa5023295\ndemo_bangumi = acer.get("https://www.acfun.cn/bangumi/aa5023295")\nprint(demo_bangumi)\n# 个人页: https://www.acfun.cn/u/39088\ndemo_up = acer.get("https://www.acfun.cn/u/39088")\nprint(demo_up)\n# 动  态: https://www.acfun.cn/moment/am2797962\ndemo_moment = acer.get("https://www.acfun.cn/moment/am2797962")\nprint(demo_moment)\n# 直  播: https://live.acfun.cn/live/378269\ndemo_live = acer.get("https://live.acfun.cn/live/378269")\nprint(demo_live)\n# 分  享: https://m.acfun.cn/v/?ac=37086357\ndemo_share = acer.get("https://m.acfun.cn/v/?ac=37086357")\nprint(demo_share)\n# 涂鸦(单页): https://hd.acfun.cn/doodle/knNWmnco.html\ndemo_doodle = acer.get("https://hd.acfun.cn/doodle/knNWmnco.html")\nprint(demo_doodle)\n```\n\n- - -\n\n\n<details>\n<summary>依赖库</summary>\n\n**依赖: 包含在 `requirements.txt` 中**\n\n+ [`httpx`](https://pypi.org/project/httpx/)`>=0.23`\n+ [`lxml`](https://pypi.org/project/lxml/)`>=4`\n+ [`beautifulsoup4`](https://pypi.org/project/beautifulsoup4/)`>=4`\n\n</details>\n\n- - - \n## 参考 & 鸣谢\n\n+ [AcFun 助手](https://github.com/niuchaobo/acfun-helper) 是一个适用于 AcFun（ acfun.cn ） 的浏览器插件。\n+ [AcFunDanmaku](https://github.com/wpscott/AcFunDanmaku) 是用C# 和 .Net 6编写的AcFun直播弹幕工具。\n+ [实现自己的AcFun直播弹幕姬](https://www.acfun.cn/a/ac16695813) [@財布士醬](https://www.acfun.cn/u/311509)\n+ QQ频道“AcFun开源⑨课”\n+ 使用 [Poetry](https://python-poetry.org/) 构建\n\n- - - \n\n## About Me\n\n[![ac彩娘-阿部高和](https://tx-free-imgs2.acfun.cn/kimg/bs2/zt-image-host/ChQwODliOGVhYzRjMTBmOGM0ZWY1ZRCIzNcv.gif)][dolacfun]\n[♂ 整点大香蕉🍌][acfunsdk_page]\n<img alt="AcFunCard" align="right" src="https://discovery.sunness.dev/39088">\n\n- - - \n\n[dolacfun]: https://www.acfun.cn/u/39088\n[acfunsdk_page]: https://www.acfun.cn/a/ac37416587\n\n[acfun.cn]: https://www.acfun.cn/\n[Issue]: https://github.com/dolaCmeo/acfunsdk/issues\n[python]: https://www.python.org/downloads/\n[venv]: https://docs.python.org/zh-cn/3.8/library/venv.html\n\n[acer]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/acer_demo.py\n[index]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/index_reader.py\n[channel]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/channel_reader.py\n[search]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/seach_reader.py\n\n[bangumi]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/bangumi_demo.py\n[video]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/video_demo.py\n[article]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/article_demo.py\n[album]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/album_demo.py\n[member]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/member_demo.py\n[moment]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/moment_demo.py\n[live]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/live_demo.py\n\n[saver]: https://github.com/dolaCmeo/acfunsdk/blob/main/examples/AcSaver_demo.py\n',
    'author': 'dolacmeo',
    'author_email': 'dolacmeo@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://pypi.org/project/acfunsdk/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# coding=utf-8
import json
from typing import Literal
from acfunsdk.source import scheme, domains, routes, apis

__author__ = 'dolacmeo'


class MyFansClub:

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        return f"{routes['medallist']}"

    def medal_list(self):
        api_req = self.acer.client.post(apis['live_medal_list'])
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data.get("medalList")

    def medal_info(self, uid: [str, int]):
        api_req = self.acer.client.post(apis['live_medal'], params={"uperId": uid})
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def medal_wear(self, uid: [str, int], on_off: bool):
        param = {"uperId": uid}
        on_off = "on" if on_off is True else "off"
        api_req = self.acer.client.post(apis[f"live_medal_wear_{on_off}"], params=param)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data


class MyFollow:

    def __init__(self, acer):
        self.acer = acer
        pass

    @property
    def referer(self):
        return f"{routes['member_following']}"

    def groups(self):
        api_req = self.acer.client.get(apis['follow_groups'])
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data.get('groupList', [])
        return None

    def _follow_group_action(self, form_data: dict):
        api_req = self.acer.client.post(apis['follow_group'], data=form_data)
        return api_req.json().get('result') == 0

    def group_add(self, name: str):
        form_data = {"action": 4, "groupName": name}
        return self._follow_group_action(form_data)

    def group_rename(self, gid: [int, str], name: str):
        form_data = {"action": 6, "groupId": gid, "groupName": name}
        return self._follow_group_action(form_data)

    def group_remove(self, gid: [int, str]):
        form_data = {"action": 5, "groupId": gid}
        return self._follow_group_action(form_data)

    def add(self, uid, attention: [bool, None] = None):
        form_data = {"toUserId": uid, "action": 1}
        if attention is True:
            form_data['action'] = 14
        elif attention is False:
            form_data['action'] = 15
        if form_data['action'] == 1:
            form_data['groupId'] = 0
        api_req = self.acer.client.post(apis['follow'], data=form_data)
        return api_req.json().get('result') == 0

    def remove(self, uid):
        form_data = {"toUserId": uid, "action": 2}
        api_req = self.acer.client.post(apis['follow'], data=form_data)
        return api_req.json().get('result') == 0

    def my_fans(self, page: int = 1, limit: int = 10, obj: bool = False):
        form_data = {"page": page, "count": limit, "action": 8}
        api_req = self.acer.client.post(apis['follow_fans'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        fans = api_data.get('friendList', [])
        if obj is True:
            return [self.acer.acfun.AcUp(x) for x in fans]
        return fans


class MyFavourite:
    folders = list()
    default_fid = None

    def __init__(self, acer):
        self.acer = acer
        self.video_groups()

    @property
    def referer(self):
        return f"{routes['member_favourite']}"

    def add(self, obj_id: str, rtype: int, fids: [str, None] = None):
        form_data = {"resourceId": obj_id, "resourceType": rtype}
        if fids is not None or rtype == 9:
            form_data['addFolderIds'] = str(fids or self.default_fid)
        req = self.acer.client.post(apis['favorite_add'], data=form_data,
                                    headers={"referer": routes['index']})
        return req.json().get('result') == 0

    def cancel(self, obj_id: str, rtype: int, fids: [str, None] = None):
        form_data = {"resourceId": obj_id, "resourceType": rtype}
        if fids is not None or rtype == 9:
            form_data['delFolderIds'] = fids or self.default_fid
        req = self.acer.client.post(apis['favorite_remove'], data=form_data,
                                    headers={"referer": routes['index']})
        return req.json().get('result') == 0

    def video_groups(self):
        api_req = self.acer.client.get(apis['favorite_list'])
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.folders = api_data.get('dataList', [])
            self.default_fid = self.folders[0].get('folderId')
            return self.folders
        return None

    def video_group_add(self, name: str):
        form_data = {"name": name}
        api_req = self.acer.client.post(apis['favorite_group_add'], data=form_data,
                                        headers={"referer": routes['index']})
        return api_req.json().get('result') == 0

    def video_group_rename(self, fid: [int, str], name: str):
        form_data = {"folderId": fid, "name": name}
        api_req = self.acer.client.post(apis['favorite_group_update'], data=form_data,
                                        headers={"referer": routes['index']})
        return api_req.json().get('result') == 0

    def video_group_remove(self, fid: [int, str]):
        form_data = {"folderId": fid}
        api_req = self.acer.client.post(apis['favorite_group_delete'], data=form_data,
                                        headers={"referer": routes['index']})
        return api_req.json().get('result') == 0

    def video_list(self, fid: [int, str], page: int = 1, limit: int = 10):
        form_data = {"folderId": fid, "page": page, "perpage": limit}
        api_req = self.acer.client.post(apis['favorite_video'], data=form_data)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None

    def article_list(self, page: int = 1, limit: int = 10):
        form_data = {"page": page, "perpage": limit}
        api_req = self.acer.client.post(apis['favorite_article'], data=form_data)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None

    def bangumi_list(self, page: int = 1, limit: int = 10):
        param = {"page": page, "perpage": limit}
        api_req = self.acer.client.get(apis['favorite_article'], params=param)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None

    def album_list(self, page: int = 1, limit: int = 10):
        form_data = {"page": page, "perpage": limit}
        api_req = self.acer.client.post(apis['favorite_album'], data=form_data)
        data = api_req.json()
        if data.get('result') == 0:
            return data.get('favoriteList', [])
        return None


class MyAlbum:

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        return f"{routes['member_album']}"

    def list(self, page: int = 1, size: int = 10):
        param = {"size": size, "page": page, "status": 0, "sort": 0}
        api_req = self.acer.client.get(apis["my_album_list"], params=param)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def add(self, title: str, rtype: Literal[2, 3], cover: str, intro: str, status: Literal[1, 2]):
        form = {
            "title": title,
            "resourceType": rtype,
            "coverImage": cover,
            "intro": intro,
            "status": status,
        }
        api_req = self.acer.client.post(apis["my_album_add"], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data.get("arubamuId")

    def remove(self, album_id: [str, int]):
        api_req = self.acer.client.post(apis["my_album_del"], data={"arubamuId": album_id})
        api_data = api_req.json()
        return api_data.get("result") == 0

    def update(self, album_id: [str, int], title: str, rtype: Literal[2, 3],
               cover: str, intro: str, status: Literal[1, 2]):
        form = {
            "arubamuId": album_id,
            "title": title,
            "resourceType": rtype,
            "coverImage": cover,
            "intro": intro,
            "status": status,
        }
        api_req = self.acer.client.post(apis["my_album_update"], data=form)
        api_data = api_req.json()
        return api_data.get("result") == 0

    def get_contents(self, album_id: [str, int], page: int = 1, size: int = 10):
        param = {"arubamuId": album_id, "page": page, "size": size}
        api_req = self.acer.client.get(apis["my_album_contents"], params=param)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def contents_add(self, album_id: [str, int], rtype: Literal[2, 3], rids: [str, list]):
        if isinstance(rids, str):
            rids = rids.split(',')
        rlist = [{"resourceId": x, "resourceType": rtype} for x in rids]
        form = {
            "arubamuId": album_id,
            "resourceListStr": json.dumps(rlist, separators=(',', ':'))
        }
        api_req = self.acer.client.post(apis["my_album_content_add"], data=form)
        api_data = api_req.json()
        return api_data.get("result") == 0

    def contents_del(self, album_id: [str, int], rids: [str, list]):
        if isinstance(rids, str):
            rids = rids.split(',')
        form = {
            "arubamuId": album_id,
            "arubamuContentIdList": rids
        }
        api_req = self.acer.client.post(apis["my_album_content_del"], data=form)
        api_data = api_req.json()
        return api_data.get("result") == 0


class MyContribute:

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        return f"{routes['up_index']}"

    def _get_my_posted(self,
                       rtype: int,
                       page: int = 1,
                       status: str = 'all',
                       sortby: str = 'recently',
                       keyword: [str, None] = None):
        # 我的文章
        status_list = {
            'all': 0,  # 全部
            'passed': 1,  # 已通过
            'posting': 2,  # 发布中
            'returned': 3,  # 已退回
        }
        sort_list = {
            'viwed': 1,  # 最多阅读
            'banana': 2,  # 最多香蕉
            'recently': 3,  # 最新发布
        }
        form_data = {
            "pcursor": 0 if (page - 1) < 0 else (page - 1),
            "resourceType": rtype,  # 2 视频 3 文章
            "authorId": self.acer.uid,
            "sortType": sort_list.get(sortby, 3),
            "status": status_list.get(status, 0),
            "keyword": keyword,
        }
        api_req = self.acer.client.post(apis['member_posted'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data.get('feed', [])
        return None

    def my_articles(self, page: int = 1, status: str = 'all', sortby: str = 'recently', keyword: [str, None] = None):
        return self._get_my_posted(3, page, status, sortby, keyword)

    def my_videos(self, page: int = 1, status: str = 'all', sortby: str = 'recently', keyword: [str, None] = None):
        return self._get_my_posted(2, page, status, sortby, keyword)

    def data_center(self, days: int = 1):
        api_req = self.acer.client.post(apis['dataCenter_overview'], data={'days': 1 if days < 1 else days})
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data
        return None

    def data_center_detail(self, rtype: str, days: int = 1, sortby: str = 'viewCount'):
        rtypes = {'video': 2, 'article': 3, 'live': None}
        assert rtype in rtypes.keys()
        sort_list = {
            "viewCount": "阅读量",
            "commentCount": "评论量",
            "stowCount": "收藏量",
            "shareCount": "分享量",
            "bananaCount": "投蕉量"
        }
        assert sortby in sort_list
        form_data = {'days': 1 if days < 1 else days}
        if rtype == 'live':
            api_req = self.acer.client.post(apis['dataCenter_live'], data=form_data)
        else:
            form_data.update({"resourceType": rtypes.get(rtype), "orderBy": sortby})
            api_req = self.acer.client.post(apis['dataCenter_content'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            return api_data
        return None

    def get_live_config(self):
        param = {
            "kpn": "ACFUN_APP",
            "kpf": "PC_WEB",
            "userId": self.acer.uid,
            "subBiz": "mainApp"
        }
        param = self.acer.update_token(param)
        api_req = self.acer.client.post(apis['live_obs_config'], params=param,
                                        headers={'referer': f"{scheme}://{domains['user']}"})
        return api_req.json()

    def post_article(self):
        # 发文章
        pass

    def post_video(self):
        # 发视频
        pass


class MyDanmaku:

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        return f"{routes['danmaku_manage']}"

    def advance_config(self):
        api_req = self.acer.client.post(apis['danmaku_config'])
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def advance_setup(self, n: Literal['1', '2', '3', '4', 1, 2, 3, 4]):
        """
        https://member.acfun.cn/inter-active/danmaku-setting
        1 任何人
        2 仅限粉丝
        3 仅限互相关注
        4 关闭高级弹幕
        """
        api_req = self.acer.client.post(apis['danmaku_setup'], data={"availableType": str(n)})
        api_data = api_req.json()
        return api_data.get("result") == 0

    def forbidden_words(self):
        api_req = self.acer.client.post(apis['ban_danmaku'])
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data.get("data")

    def forbidden_add(self, words: str):
        api_req = self.acer.client.post(apis['ban_danmaku_add'], data={"forbiddenWords": words})
        api_data = api_req.json()
        return api_data.get("result") == 0

    def forbidden_del(self, words: str):
        api_req = self.acer.client.post(apis['ban_danmaku_del'], data={"forbiddenWords": words})
        api_data = api_req.json()
        return api_data.get("result") == 0

    def advance_list(self,
                     page: int = 1,
                     ac_num: [int, str, None] = None,
                     vid: [int, str, None] = None,
                     search: [str, None] = None):
        form = {
            "dougaId": ac_num or "",
            "videoId": vid or "",
            "danmakuText": search or "",
            "page": page,
        }
        api_req = self.acer.client.post(apis['search_danmaku_adv'], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def danmaku_list(self,
                     page: int = 1,
                     ac_num: [int, str, None] = None,
                     vid: [int, str, None] = None,
                     search: [str, None] = None):
        form = {
            "dougaId": ac_num or "",
            "videoId": vid or "",
            "danmakuText": search or "",
            "page": page,
        }
        api_req = self.acer.client.post(apis['search_danmaku'], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def danmaku_protect(self, ac_num: [int, str], danmaku_id: [int, str], on_off: bool = True):
        form = {
            "videoResourceType": "douga",
            "videoResourceId": ac_num,
            "danmakuId": danmaku_id,
            "rank": 9 if on_off is True else 5,
        }
        api_req = self.acer.client.post(apis['protect_danmaku'], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def danmaku_del(self, danmaku_ids: [int, str]):
        form = {"danmakuIdList": str(danmaku_ids)}
        api_req = self.acer.client.post(apis['delete_danmaku'], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data


class BananaMall:

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        return f"{routes['member_mall']}"

    def shop_list(self, page: int = 1, size: int = 30, stype: Literal[1, 2, 3] = 1, asc: bool = False):
        form = {"pageNo": page, "pageSize": size, "sortType": stype, "asc": asc}
        api_req = self.acer.client.post(apis['shop_list'], data=form)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def my_items(self, page: int = 1, size: int = 30):
        param = {"page": page, "count": size}
        api_req = self.acer.client.get(apis['shop_user_item'], params=param)
        api_data = api_req.json()
        assert api_data.get("result") == 0
        return api_data

    def set_item(self, pid: int, on_off: bool = True):
        form = {"productId": pid}
        on_off = "use" if on_off is True else "unuse"
        api_req = self.acer.client.post(apis[f"shop_user_item_{on_off}"], data=form)
        api_data = api_req.json()
        return api_data.get("result") == 0

    def my_orders(self):
        # https://www.acfun.cn/member/mall?tab=orders
        # 缺少足够样本获取翻页接口请求
        raise PermissionError("接口未知：缺少足够样本获取翻页接口请求")



# coding=utf-8
import re
import base64
from uuid import uuid4
from datetime import timedelta
from acfunsdk.source import routes, apis

__author__ = 'dolacmeo'


class B64s:
    STANDARD = b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
    EN_TRANS = STANDARD
    DE_TRANS = STANDARD

    def __init__(self, s: [bytes, bytearray], n: [int, None] = None):
        self.raw = s
        if isinstance(n, int):
            n = n % 64
            new = self.STANDARD[n:] + self.STANDARD[:n]
            self.EN_TRANS = bytes.maketrans(self.STANDARD, new)
            self.DE_TRANS = bytes.maketrans(new, self.STANDARD)

    def b64encode(self):
        return base64.standard_b64encode(self.raw).translate(self.EN_TRANS)

    def b64decode(self):
        return base64.b64decode(self.raw.translate(self.DE_TRANS))


def sizeof_fmt(num, suffix="B"):
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return f"{num:3.1f} {unit}{suffix}"
        num /= 1024.0
    return f"{num:.1f} Yi{suffix}"


def match_info(text):
    rex = re.compile(r"(?P<title>.*)\r"
                     r"UP:(?P<up>.*)\r"
                     r"发布于(?P<createTime>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2})"
                     r"\xa0/\xa0点击:(?P<viewCount>\d+)"
                     r"\xa0/\xa0评论:(?P<commentCount>\d+)")
    result = rex.fullmatch(text)
    if result is None:
        return None
    return result.groupdict()


def match1(text, *patterns):
    """Scans through a string for substrings matched some patterns (first-subgroups only).

    Args:
        text: A string to be scanned.
        patterns: Arbitrary number of regex patterns.

    Returns:
        When only one pattern is given, returns a string (None if no match found).
        When more than one pattern are given, returns a list of strings ([] if no match found).
    """

    if len(patterns) == 1:
        pattern = patterns[0]
        match = re.search(pattern, text)
        if match:
            return match.group(1)
        else:
            return None
    else:
        ret = []
        for pattern in patterns:
            match = re.search(pattern, text)
            if match:
                ret.append(match.group(1))
        return ret


def matchall(text, patterns):
    """Scans through a string for substrings matched some patterns.

    Args:
        text: A string to be scanned.
        patterns: a list of regex pattern.

    Returns:
        a list if matched. empty if not.
    """

    ret = []
    for pattern in patterns:
        match = re.findall(pattern, text)
        ret += match

    return ret


def image_uploader(client, image_data: bytes, ext: str = 'jpeg'):
    token_req = client.post(apis['image_upload_gettoken'], data=dict(fileName=uuid4().hex.upper() + f'.{ext}'))
    token_data = token_req.json()
    assert token_data.get('result') == 0
    resume_req = client.get(apis['image_upload_resume'], params=dict(upload_token=token_data['info']['token']))
    resume_data = resume_req.json()
    assert resume_data.get('result') == 1
    fragment_req = client.post(apis['image_upload_fragment'], data=image_data,
                               params=dict(upload_token=token_data['info']['token'], fragment_id=0),
                               headers={"Content-Type": "application/octet-stream"})
    fragment_data = fragment_req.json()
    assert fragment_data.get('result') == 1
    complete_req = client.post(apis['image_upload_complete'],
                               params=dict(upload_token=token_data['info']['token'], fragment_count=1))
    complete_data = complete_req.json()
    assert complete_data.get('result') == 1
    result_req = client.post(apis['image_upload_geturl'], data=dict(token=token_data['info']['token']))
    result_data = result_req.json()
    assert result_data.get('result') == 0
    return result_data.get('url')


def limit_string(s, n):
    s = s[:n]
    if len(s) == n:
        s = s[:-2] + "..."
    return s


def thin_string(_string: str, no_break: bool = False):
    final_str = list()
    for line in _string.replace('\r', '').split('\n'):
        new_line = ' '.join(line.split()).strip()
        if len(new_line):
            final_str.append(new_line)
    if no_break is True:
        return " ".join(final_str)
    return " ↲ ".join(final_str)


def warp_mix_chars(_string: str, lens: int = 40, border: [tuple, None] = None):
    output = list()
    tmp_string = ""
    tmp_count = 0
    for i, _char in enumerate(_string):
        char_count = 2 if '\u4e00' <= _char <= '\u9fcc' else 1
        if tmp_count + char_count > lens:
            tmp_string += ' ' * (lens - tmp_count)
            output.append(tmp_string)
            tmp_string = ""
            tmp_count = 0
        tmp_string += _char
        tmp_count += char_count
    if tmp_count != 0:
        tmp_string += ' ' * (lens - tmp_count)
        output.append(tmp_string)
    if isinstance(border, tuple) and len(border) == 2:
        output = [f"{border[0]}{x}{border[1]}" for x in output]
    return output


def get_channel_info(html):
    rex = re.compile(r'\{subChannelId:(\d+),subChannelName:\"((?:(?!").)*)\"}')
    result = rex.findall(html)
    if result is not None:
        return {"subChannelId": result[0][0], "subChannelName": result[0][1]}
    return {}


def ms2time(ms: int):
    d = timedelta(milliseconds=ms)
    return str(d).split('.')[0]


def get_page_pagelets(page_obj):
    data = list()
    for item in page_obj.select("[id^=pagelet_]"):
        data.append(item.attrs['id'])
    if len(data):
        data.append('footer')
    return data


def url_complete(url):
    if isinstance(url, str):
        if url.startswith('//'):
            url = f"https:{url}"
        elif not url.startswith('http'):
            url = f"{routes['index']}{url}"
    return url
